;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; HUD routines (CE only)                                        ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; loadHud - load the HUD bar image to sbuf
;   input:  none
;   output: none
;------------------------------------------------
loadHud:
        ; duplicate the grayscale part of the palette, with black at the start
        ld      hl,mpLcdPalette+2
        ld      de,mpLcdPalette+(64*2)
        ld      bc,2
        ldir
        ld      hl,mpLcdPalette+(17*2)
        ld      bc,2*15
        ldir

        ; unpack
        ld      hl,(pDatFile)
        ld      de,hudbar
        add     hl,de
        ld      de,sbuf
        ld      ix,huffTree
        jp      huffExtr

;------------------------------------------------
; showHud - draw the empty HUD
;   input:  none
;   output: none
;------------------------------------------------
showHud:
        ld      a,(p1p)
        or      a
        jr      z,showHudSkip1
        ld      hl,(pVbuf)
        ld      de,14*320
        add     hl,de
        ld      de,sbuf
        ex      de,hl
        ld      bc,9*320
        ldir

        ld      a,1
        ld      l,15
        ld      ix,p1x
        call    updateHud

showHudSkip1:
        ld      a,(p2p)
        or      a
        ret     z
        ld      hl,(pVbuf)
        ld      de,217*320
        add     hl,de
        ld      de,sbuf
        ex      de,hl
        ld      bc,9*320
        ldir

        ld      a,2
        ld      l,218
        ld      ix,p2x
        ; fall through to updateHud

;------------------------------------------------
; updateHud - update the HUD for a player
;   input:  A = player #
;           IX => player vars
;           L = y pos to use
;   output: none
;------------------------------------------------
updateHud:
        ; show player #
        ld      de,13
        call    showHudDigit

        ; show health
        ld      a,(ix+PH)
        ld      b,1
        ld      de,39
        call    showHudBar

        ; show lives
        ld      a,(ix+PL)
        ld      de,149
        call    showHudNumber

        ; show ammo clips
        ld      a,(ix+PA+1)
        ld      de,203
        call    showHudNumber

        ; show ammo clip level
        ld      a,(ix+PA)
        ld      b,3
        ld      de,221
        call    showHudBar

        ; show keys
        ld      a,(ix+PK)
        ld      de,291
        ; fall through to showHudNumber

;------------------------------------------------
; showHudNumber - show a value in the HUD font
;   input:  A = value (max 99)
;           DE = x pos
;           L = y pos
;   output: DE = x pos after digit
;           L = y pos
;------------------------------------------------
showHudNumber:
        cp      100
        jr      c,shnStart
        bit     7,a
        jr      nz,shnZero
        ld      a,99                            ; if value > 99, max out at 99
        jr      shnStart
shnZero:
        xor     a                               ; if value < 0, set to 0 (used to avoid showing lives 255 when player runs out of lives)
shnStart:
        push    de
        push    hl
        LDHLA()
        ld      a,10
        call    _divHLbyA
        ld      (__shnDigit2),a
        ld      a,l
        pop     hl
        pop     de
        call    showHudDigit
__shnDigit2             = $+1
        ld      a,$00
        ; fall through to showHudDigit

;------------------------------------------------
; showHudDigit - show a digit in the HUD font
;   input:  A = digit value
;           DE = x pos
;           L = y pos
;   output: DE = x pos after digit
;           L = y pos
;   notes:  IX preserved
;------------------------------------------------
showHudDigit:
        push    ix
        push    hl
        push    de
        ld      ix,fonthud
        ld      c,a
        ld      b,8
        mlt     bc
        add     ix,bc                           ; IX => sprite
        ld      h,160
        mlt     hl
        add     hl,hl
        add     hl,de
        ld      de,(pVbuf)
        add     hl,de                           ; HL => where to draw character
        ld      bc,8*256+128                    ; B = 8 rows, C = %10000000 bitmask
        ld      de,320-8
shdLoop:
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        ld a,(ix) \ and c \ jr z,$+4 \ ld (hl),COLOUR_WHITE \ rrc c \ inc hl
        add     hl,de
        inc     ix
        djnz    shdLoop
        pop     de
        ld      hl,8
        add     hl,de
        ex      de,hl
        pop     hl
        pop     ix
        ret

;------------------------------------------------
; showHudBar - show a HUD bar
;   input:  A = value to use to calc width
;           B = # of times to divide by 2 to calc width
;           DE = x pos
;           L = y pos
;   output: L = y pos
;------------------------------------------------
showHudBar:
        srl     a
        djnz    showHudBar
        ret     z                               ; if value is now 0, nothing to show
        push    ix
        push    hl
        ld      b,a                             ; B = number of columns to draw
        ld      h,160
        mlt     hl
        add     hl,hl
        add     hl,de
        ld      de,(pVbuf)
        add     hl,de                           ; HL => where to start drawing
        ld      ix,hudline
        ld      de,320
shbLoop:
        push    hl
        ld a,(ix+0) \ ld (hl),a \ add hl,de
        ld a,(ix+1) \ ld (hl),a \ add hl,de
        ld a,(ix+2) \ ld (hl),a \ add hl,de
        ld a,(ix+3) \ ld (hl),a \ add hl,de
        ld a,(ix+4) \ ld (hl),a \ add hl,de
        ld a,(ix+5) \ ld (hl),a \ add hl,de
        ld a,(ix+6) \ ld (hl),a \ add hl,de
        pop     hl
        inc     hl
        djnz    shbLoop
        pop     hl
        pop     ix
        ret

.end
