;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Info box routines                                             ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; showInfoBox - show an info box and "type" out a page of text
;   input:  HL => title to use for info box
;           DE => page string data to be printed using displayPage routine
;           A = 0 if normal info box, otherwise A<>0 is a prompt box
;   output: HL => next byte of data after end of page data
;           CA = Yes selected, NC = No selected (prompt box only)
;------------------------------------------------
showInfoBox:
        push    af                              ; save info box type
        push    de                              ; save string pointer for content
        push    hl                              ; save string pointer for title
        ; erase the majority of the screen
        ld      b,64-16
        ld      hl,gbuf+(8*12)+1
        xor     a
infoBoxClear:
        push    bc
        push    hl
        call    infoBoxHline
        pop     hl
        pop     bc
        ld      de,12
        add     hl,de                           ; HL => next line
        djnz    infoBoxClear
        ; draw the horizontal borders
        ld      a,$FF
        ld      hl,gbuf+(7*12)+1
        call    infoBoxHline
        ld      hl,gbuf+(56*12)+1
        call    infoBoxHline
        ; draw the vertical borders
        ld      hl,gbuf+(7*12)
        ld      c,%00000001
        call    infoBoxVline
        ld      hl,gbuf+(7*12)+11
        ld      c,%10000000
        call    infoBoxVline
        ; print text
        pop     hl                              ; HL => title string
        call    showStrings
        ld      hl,gbuf+(8*12)+1
        ld      bc,8*256+10                     ; B = 8 lines to invert, C = 10 bytes per line
        call    invertPartLine                  ; invert the "NEW ACHIEVEMENT!" text
        pop     hl                              ; HL => content string data
        call    displayPageNoClear
        ; wait for [ENTER]
        pop     af
        or      a                               ; normal info box or prompt box?
        jr      nz,promptBox
        ld      hl,strInfo2nd
        call    showStrings
        bcall(_grbufcpy)
        call    wait2ndEnter
        ld      b,INTO_GAME_DELAY
        jp      delay
promptBox:
        ld      hl,strInfoPrompt
        call    showStrings
        bcall(_grbufcpy)
promptBoxLoop:
        bcall(GET_KEY)
        cp      GK_2ND                          ; [2nd]?
        jr      z,promptBoxYes
        cp      GK_ENTER                        ; [ENTER]?
        jr      z,promptBoxYes
        cp      GK_MODE                         ; [MODE]?
        jr      z,promptBoxNo
        cp      GK_CLEAR                        ; [CLEAR]?
        jr      nz,promptBoxLoop
promptBoxNo:
        or      a                               ; clear carry
        ret
promptBoxYes:
        scf                                     ; set carry
        ret

;------------------------------------------------
; infoBoxHline - draw/erase an 80 pixel long horizontal line for the info box
;   input:  HL => byte of gbuf to start at
;           A = value to write
;   output: A = value to write
;------------------------------------------------
infoBoxHline:
        push    hl
        pop     de
        inc     de                              ; DE => byte after (HL)
        ld      bc,9
        ld      (hl),a
        ldir
        ret

;------------------------------------------------
; infoBoxVline - draw a 50 pixel long vertical line for the info box
;   input:  HL => byte of gbuf to start at
;           C = bitmask to use
;   output: none    
;------------------------------------------------
infoBoxVline:
        ld      b,50
        ld      de,12
ibvlLoop:
        ld      a,(hl)
        or      c
        ld      (hl),a
        add     hl,de
        djnz    ibvlLoop
        ret

.end
