;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Level routines                                                ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; loadLevel - load current level
;   input:  none
;   output: none
;------------------------------------------------
loadLevel:
        ld      hl,miscFlags
        res     loadingState,(hl)               ; clear the loading state flag
        xor     a
        ld      hl,counters
        ld      de,counters+1
        ld      bc,7
        ld      (hl),a
        ldir                                    ; clear event useable counters
        ld      (state),a                       ; make sure level is started in normal game state
        ld      (shake),a                       ; reset the shake counter
        ld      (p1dd),a
        ld      (p2dd),a
        ld      (p1i),a
        ld      (p2i),a
        ld      (p1ht),a
        ld      (p2ht),a
        ld      (p1sh),a
        ld      (p2sh),a
        ld      (numPickups),a
        ld      (numDoors),a
        ld      hl,0
        ld      (tileShotPtr),hl                ; reset ptr to previously shot tile in level data
        inc     a                               ; A = 1
        ld      (aflagAliens),a                 ; set flag to denote that no aliens have been shot during this level
        ld      a,99
        ld      (time),a

        call    clearAnimTable                  ; clear animTable
        call    clearBulletTable                ; clear bulletTable
        call    clearAllEnemyTables             ; clear enemyTable, aholeTable
        call    clearEbullet                    ; clear ebullet
        call    clearObjectTable                ; clear objectTable

#ifdef TI83PLUS
        call    iniLevelLoad                    ; copy level loading routines to ram
#endif
        ; get ptr to compressed level data
#ifdef TI83PLUS
        ld      a,(episode)
        LDHLA()
        ld      de,episodeTable
        add     hl,de
        ld      l,(hl)
        ld      h,0
        add hl,hl \ add hl,hl
        ex      de,hl                           ; DE = offset to start of level entries for this episode in levelTable
        ld      hl,(level)
        ld      h,0
        add hl,hl \ add hl,hl                   ; HL = level*4
        add     hl,de                           ; HL = offset to level entry
        ld      de,levelTable
        add     hl,de                           ; HL => 4 bytes of level location data [appPage,NULL,addressLSB,addressMSB]
        ld      c,(hl)
        ld      b,0
        inc hl \ inc hl
        LDHLIND()                               ; HL => compressed level data (once the right app page is loaded)
        ld      de,(tempFile)
        ; call routine in ram, which will swap to correct rom page, decompress the level data to ram, and revert to this rom page
        call    ram_huffExtr
#endif
#ifdef TI84PLUSCE
        call    getEpLvlOffset
        ld      de,levelTable
        add     hl,de
        ld      de,(hl)                         ; DE = offset to compressed file in data appvar
        ld      hl,(pLvlFile)
        add     hl,de                           ; HL => compressed level data
        ld      de,(tempFile)                   ; DE => where to decompress level to
        ld      ix,huffTree                     ; IX => 1024 of temp mem for huffExtr
        call    HuffExtr
        ; reload default palette
        call    loadDefaultPalette
        ; parse the mission objectives to move the text for the larger screen
        ld      hl,(tempFile)
        call    modifyText
#endif

        call    showObjectives                  ; display mission objectives

        ld      de,levelType                    ; DE => where to store level stats
        ld      bc,LEVEL_HEADER                 ; BC = number of bytes to copy
        ldir                                    ; copy level stats
        ld      (tileDataPtr),hl                ; save pointer to tile data
        
#ifdef TI84PLUSCE
        ; check if passcode should be shown
        push    hl
        ld      hl,levelFlags
        bit     LFLAG_PASSCODE,(hl)
        jr      z,afterShowObjPasscode
        ld      hl,miscFlags
        bit     passcodeUsed,(hl)
        jr      nz,afterShowObjPasscode
        call    makePasscode
        ld      hl,passcode
        ld      de,strPasscode+17
        ld      bc,10
        ldir
        ld      hl,strPasscode
        call    displayPageNoClear
afterShowObjPasscode:
        pop     hl
#endif

        ; get pointer to start of events
        push    hl
        ld      hl,(levelWidth)                 ; L = width, H = height
#ifdef TI83PLUS
        bcall(_htimesl)
#endif
#ifdef TI84PLUSCE
        mlt     hl
#endif
        pop     de
        add     hl,de                           ; HL => start of event data
        ld      a,(hl)                          ; A = number of events
        ld      (numEvents),a
        or      a                               ; are there any events in this level?
        call    nz,iniEvents                    ; if so, set pointers

        ; load tileset (and palette and reset palShift on 84CE)
        call    loadLevelGfx
#ifdef TI84PLUSCE
        ; load enemy sprites
        call    loadEnemySprites
#endif

#ifdef TI83PLUS
        ; draw the start of the loading bar
        ld      hl,(levelHeight)
        dec     l
        ld      a,95
        sub     l
loadingBar:
        push    af
        ld      hl,gbuf+768-12
        call    drawPixel
        pop     af
        push    af
        ld      hl,gbuf+768-24
        call    drawPixel
        pop     af
        dec     a
        cp      $FF
        jr      nz,loadingBar
#endif

        ld      a,(levelType)
        and     $0F
        LDHLA()
        push    hl
        ld      de,tilesetWallTable
        add     hl,de
        ld      a,(hl)
        rlca                                    ; bit 1 = wall calc flag
        pop     hl
        ld      de,tilesetFloorTable
        add     hl,de
        or      (hl)                            ; bit 0 = floor calc flag
        ld      (tempB),a                       ; save flags

        ; calculate all floor & wall tiles for levels with a tileset that require it
        xor     a
        ld      (tempD),a
        ld      ix,(tileDataPtr)
        ld      a,(levelHeight)
        dec     a
        ld      c,a                             ; C = y to start at (bottom of level, and work back up)
calcLevelLoopRow:
        ld      a,(levelWidth)
        dec     a
        ld      b,a                             ; B = x to start at (far right of level, and work back to the left)
calcLevelLoopCol:
        ld      a,(tempB)
        bit     0,a
        jr      z,skipCalcFloors
        ld      hl,tempD
        ld      a,(ix)
        cp      TILE_FLOOR
        jr      c,skipCalcFloors
        cp      TILE_FLOOR+4
        jr      nc,skipCalcFloors
        add     a,(hl)
        ld      (ix),a
        ld      a,(hl)
        inc     a
        and     $03
        ld      (hl),a
skipCalcFloors:
        push    bc                              ; save x/y pos
        ld      a,(tempB)
        bit     1,a
        jr      z,endCLLoop
        bcall(ab5_checkWall)                    ; check tile to see if it's a wall
        jr      nc,endCLLoop                    ; if not, go to next tile
        ld      (wallTilePtr),hl                ; save ptr to tile in tile data
        xor     a
        ld      (wallFlags),a                   ; reset wall flags (one bit for each tile surrounding the current tile)
        ld      de,wallMaskTable                ; DE => mask table to use for wall flags
        ld      hl,wallOffsetTable              ; HL => coord offset table to find all surrounding tile positions
        ld      a,8                             ; A = 8 surrounding tiles to check
findSurroundingWallsLoop:
        pop     bc                              ; get our centre x/y pos
        push    bc                              ; .. and put it back on the stack for later
        push    af                              ; save the loop counter
        push    de                              ; save mask table ptr
        ld      a,b
        add     a,(hl)                          ; apply x offset
        ld      b,a                             ; B = x pos to check
        inc     hl
        ld      a,c
        add     a,(hl)                          ; apply y offset
        ld      c,a                             ; C = y pos to check
        inc     hl
        push    hl                              ; save offset table ptr
        bcall(ab5_checkWall)                    ; check tile to see if it's a wall
        pop     hl                              ; restore offset table ptr
        pop     de                              ; restore mask table ptr
        jr      nc,endFSWLoop
        ld      a,(de)                          ; A = bit mask to OR
        ld      bc,(wallFlags)
        or      c                               ; apply it with the flags byte
        ld      (wallFlags),a                   ; save updated flags
endFSWLoop:
        inc     de                              ; update mask table ptr
        pop     af                              ; restore loop counter
        dec     a
        jr      nz,findSurroundingWallsLoop     ; loop until it reaches 0

        ld      a,(wallFlags)                   ; get flag byte
        and     $0F                             ; first we want to deal with 4 lsb
        ld de,0 \ ld e,a
        ld      hl,wallTable
        add     hl,de                           ; HL => offset value for tile
        ld      a,(hl)                          ; A = offset value
        ld      hl,(wallTilePtr)                ; HL => tile in tile data (set from earlier)
        add     a,(hl)                          ; A = new tile value
        ld      (hl),a                          ; set it
        sub     TILE_COMPLEXWALL                ; check if new tile is a "complex" wall
        jr      c,endCLLoop                     ; if not, we're done processing this tile
        ; otherwise, we need to check the flags of the diagonals
        push    hl                              ; save tile data ptr
        ld de,0 \ ld e,a                        ; DE = offset for wallComplexTable
        ld      hl,wallComplexTable
        add     hl,de                           ; HL => mask to apply to check correct diagonal block flag bits
        ld      a,(wallFlags)                   ; A = flag byte
        cpl                                     ; invert it
        and     (hl)                            ; apply mask to it
        pop     hl                              ; restore tile data ptr
        jr      nz,endCLLoop                    ; if any applicable corner blocks were not walls, leave the tile alone
        ; otherwise, increase wall tile by 5 to get it's counterpart, which makes the wall tile the "more solid" version
        ld      a,5
        add     a,(hl)                          ; A = new wall tile
        ld      (hl),a                          ; save it

endCLLoop:
        inc     ix
        pop     bc                              ; restore x/y pos
        dec     b                               ; dec x pos
        bit     7,b
        jp      z,calcLevelLoopCol              ; if x hasn't gone lower than 0, loop again
#ifdef TI83PLUS
        ; update progress bar
        push    bc                              ; save y pos
        ld      a,95
        sub     c
        push    af
        ld      hl,gbuf+768-12
        call    drawPixel
        pop     af
        ld      hl,gbuf+768-24
        call    drawPixel
        bcall(_grbufcpy)
        pop     bc                              ; restore y pos
#endif

        dec     c                               ; dec y pos
        bit     7,c
        jp      z,calcLevelLoopRow              ; if y hasn't gone lower than 0, loop again

afterCalcWalls:
        ; if episode 3, check route flags
        ld      a,(episode)
        cp      2
        jr      nz,afterEp3RouteFlags
        ld      a,(level)
        LDHLA()
        ld      de,ep3RouteTable
        add     hl,de
        ld      a,(hl)
        ld      hl,routeFlags
        or      (hl)
        ld      (hl),a
____TEST = routeFlags
afterEp3RouteFlags:
#ifdef TI83PLUS
        ; clear loading bar
        ld      hl,gbuf+768-24
        ld      de,gbuf+768-23
        ld      (hl),$00
        ld      bc,23
        ldir
        ld      hl,str2nd
        call    showStrings                     ; "[2nd]"
        bcall(_grbufcpy)
        call    wait2ndEnter

        ; check if passcode should be shown
        ld      hl,levelFlags
        bit     LFLAG_PASSCODE,(hl)
        jr      z,afterShowObjPasscode
        ld      hl,miscFlags
        bit     passcodeUsed,(hl)
        jr      nz,afterShowObjPasscode
        call    makePasscode
        bcall(_grbufclr)
        ld      hl,strPasscode
        ld      de,videoBuffer
        ld      bc,42
        ldir
        ld      hl,passcode
        ld      de,videoBuffer+13
        ld      bc,10
        ldir
        ld      hl,strPasscodeTitle
        ld      de,videoBuffer
        xor     a
        call    showInfoBox
afterShowObjPasscode:
#endif
#ifdef TI84PLUSCE
        call    wait2ndEnterSto
        cp      GK_STO
        jr      nz,startLevel
        ; [STO] was pressed, check if there is a passcode to save
        ld      hl,levelFlags
        bit     LFLAG_PASSCODE,(hl)
        jr      z,startLevel
        ld      hl,miscFlags
        bit     passcodeUsed,(hl)
        jr      nz,startLevel
        ; try to save passcode
        call    savePasscode
#endif

startLevel:
        ld      b,INTO_GAME_DELAY
        call    delay                           ; tiny delay to avoid shooting as soon as entering level
#ifdef TI83PLUS
        bcall(ab5_drawLevel)                    ; draw level and continue to main game loop
#endif
#ifdef TI84PLUSCE
        call    loadHud
        jp      drawLevel
#endif

#ifdef TI83PLUS
;------------------------------------------------
; iniLevelLoad - copy level loading routines to ram
;   input:  none
;   output: none
;------------------------------------------------
iniLevelLoad:
        ; load levelHuff to ram for execution during page swapping
        ld      hl,_ram_huffExtr
        ld      de,ram_huffExtr
        ld      bc,RAMHUFF_LEN
        ldir
        ; same for huffExtr
        ld      hl,huffExtr
        ld      de,ram_huffExtr+RAMHUFF_LEN
        ld      bc,HUFFEXTR_LEN
        ldir
        ret

;------------------------------------------------
; _ram_huffExtr - decompress a file using huffExtr from another ROM page
;   input:  B = file #
;           C = offset to app page with file data
;           HL => compressed data
;           DE => where to decompress to
;   output: none
;------------------------------------------------
_ram_huffExtr:
        in      a,(6)                           ; A = current rom page
        push    af                              ; save it
        sub     c                               ; A = rom page with level data
        out     (6),a
        ld      ix,gbuf                         ; IX => 1024 of temp mem for huffExtr
        call    ram_huffExtr+RAMHUFF_LEN        ; extract current level (using the version of huffExtr that's been loaded to ram)
        pop     af
        out     (6),a                           ; restore original rom page
        ret
RAMHUFF_LEN             = $-_ram_huffExtr
#endif

;------------------------------------------------
; clearAnimTable - clear animTable
;   input:  none
;   output: none
;------------------------------------------------
clearAnimTable:
        ld      hl,animTable                    ; HL => first byte to clear
        ld      de,animTable+1                  ; DE => where to start copy cleared bytes to
        ld      bc,NUM_ANIMS*ANIM_SIZE-1        ; BC = number of bytes to copy/clear
        ld      (hl),0                          ; clear first byte
        ldir                                    ; copy/clear the rest
        ret

;------------------------------------------------
; clearBulletTable - clear all bulletTable entries
;   input:  none
;   output: none
;------------------------------------------------
clearBulletTable:
        ld      hl,bulletTable                  ; first byte to clear
        ld      de,bulletTable+1                ; first byte to copy cleared bytes to
        ld      bc,NUM_BULLETS*BULLET_SIZE-1    ; number of bytes to clear
        ld      (hl),0                          ; clear first byte
        ldir                                    ; clear all subsequent bytes
        ret

;------------------------------------------------
; clearAllEnemyTables - clear enemyTable, aholeTable
;   input:  none
;   output: none
;------------------------------------------------
clearAllEnemyTables:
        ld      hl,enemyTable                   ; HL => first byte to clear
        ld      de,enemyTable+1                 ; DE => where to copy cleared bytes to
        ld      bc,NUM_ENEMIES*ENEMY_SIZE-1     ; BC = number of cleared bytes to copy
        ld      (hl),0                          ; clear first byte
        ldir                                    ; clear all subsequent bytes
        ld      hl,aholeTable                   ; ditto for aholeTable
        ld      de,aholeTable+1
        ld      bc,NUM_AHOLES*AHOLE_SIZE-1
        ld      (hl),0
        ldir
        ret

;------------------------------------------------
; clearEbullet - clear enemy bullet
;   input:  none
;   output: none
;------------------------------------------------
clearEbullet:
        ld      hl,ebullet
        ld      de,ebullet+1
        ld      bc,EBULLET_SIZE-1
        ld      (hl),0
        ldir
        ret

;------------------------------------------------
; clearObjectTable - clear objectTable
;   input:  none
;   output: none
;------------------------------------------------
clearObjectTable:
        ld      hl,objectTable
        ld      de,objectTable+1
        ld      bc,NUM_OBJECTS*OBJECT_SIZE-1
        ld      (hl),0
        ldir
        ret

#ifdef TI83PLUS
;------------------------------------------------
; loadLevelGfx - load the correct tileset
;   input:  none
;   output: none
;------------------------------------------------
loadLevelGfx:
        bcall(ab5_saveVideo)
        call    iniLevelLoad

        ; always load tileset #0 first, as it includes some tiles that all other tilesets use
        xor     a
        call    loadTileset

        ; bit of a hack, but Episode III tower tilesets always load TCV as a second base tileset
        ld      a,(levelType)
        and     $0F
        cp      6
        jr      c,skipEp3Tiles
        ld      a,5
        call    loadTileset

skipEp3Tiles:
        ; load tileset mods specific to this level
        ld      a,(levelType)
        and     $0F
        call    nz,loadTileset                  ; don't need to reload if tileset #0

        bcall(ab5_restoreVideo)
        ret

;------------------------------------------------
; loadTileset - load all segments for a tileset
;   input:  A = tileset #
;   output: none
;------------------------------------------------
loadTileset:
        ld      l,a
        ld      h,0
        add     hl,hl
        ld      de,tilesetTable
        add     hl,de
        LDHLIND()
        ld      b,(hl)                          ; B = # of mods to load
        inc     hl
        ld      c,(hl)                          ; C = first file # to unpack
        inc     hl
ltLoop:
        push    bc
        ld      b,c
        ld      e,(hl)
        inc     hl
        ld      d,(hl)
        inc     hl
        push    hl
        ld      hl,(pTiles)
        add     hl,de
        ld      de,tiles
        ex      de,hl
        ld      c,6                             ; C = app page with packed tile data
        call    ram_huffExtr
        pop     hl
        pop     bc
        inc     c
        djnz    ltLoop
        ret
#endif

.end
