;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Level routines                                                ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; drawLevel - draw the level and scroll if necessary to centre the player
;   input:  none
;   output: none
;------------------------------------------------
drawLevel:
#ifdef TI83PLUS
        pop hl \ pop hl \ pop hl                ; clear stack of bcall()
        bcall(_grbufclr)
#endif
drawLevelJp:
        ld      hl,miscFlags
        ld      a,(hl)
        and     %00110000                       ; are we loading from a save state, or teleporting?
        jr      z,findStart
        call    drawLevelResume                 ; if so, set the startX / startY positions to the player position
        jr      calcScreen
findStart:
        ; find the correct start position (based on the previous level that was finished)
        ld      ix,aStart                       ; IX => start position array
        bit     passcodeUsed,(hl)               ; HL already points to miscFlags from above
        jr      nz,startFound
        ld      b,NUM_STARTS
        ld      a,(prevLevel)                   ; A = previous level #
        ld      de,3
findStartLoop:
        cp      (ix)                            ; does level # match?
        jr      z,startFound
        add     ix,de
        djnz    findStartLoop
        ; if execution reaches here, no matching level was found, so default to the first start position to avoid crashing
        ld      ix,aStart
startFound:
        ld      a,(ix+1)
        ld      (startX),a
        ld      a,(ix+2)
        ld      (startY),a
calcScreen:
        ; calc left side of screen
        ld      a,(startX)
        sub     (TILE_COLS/2)-1
        jr      c,startDrawLeft                 ; if close to left edge of level, start drawing from left edge
        add     a,TILE_COLS-1
        ld      hl,(levelWidth)                 ; L = level width
        dec     l
        cp      l
        jr      nc,startDrawRight               ; if close to right edge of level, start drawing from right edge - 11
        sub     TILE_COLS-1                     ; otherwise, player will be centred
saveScreenX:
        ld      (screenX),a                     ; screenX is currently only stored as 1 byte block coord, later it will be converted to absolute pixel value
        ; calc top side of screen
        ld      a,(startY)
        sub     (TILE_ROWS/2)-1
        jr      c,startDrawTop                  ; if close to top edge of level, start drawing from top edge
        add     a,TILE_ROWS-1
        ld      hl,(levelHeight)                ; L = level height
        dec     l
        cp      l
        jr      nc,startDrawBottom              ; if close to bottom edge of level, start drawing from bottom edge - 7
        sub     TILE_ROWS-1                    ; otherwise, player will be centred
saveScreenY:
        ld      (screenY),a                     ; screenY is currently only stored as 1 byte block coord, later it will be converted to absolute pixel value
        jr      afterCalcScreenXY               ; skip past sub routines to next step
startDrawLeft:
        xor     a
        jr      saveScreenX
startDrawRight:
        ld      a,l                             ; A = level width
        sub     TILE_COLS-1
        jr      saveScreenX
startDrawTop:
        xor     a
        jr      saveScreenY
startDrawBottom:
        ld      a,l                             ; A = level height
        sub     TILE_ROWS-1
        jr      saveScreenY

afterCalcScreenXY:
#ifdef TI83PLUS
        ; start drawing level
        xor     a
        ld      (dlY),a                         ; reset y offset / counter
drawLevelNextY:
        xor     a
        ld      (dlX),a                         ; reset x offset / counter
        ld      a,(dlY)
        ld      hl,screenY
        add     a,(hl)                          ; A = top of screen y + offset
        ld      hl,(levelWidth-1)               ; H = level width
        ld      l,a                             ; L = top of screen y + offset
        bcall(_htimesl)                         ; HL = offset to row in tile data
        ld      de,(tileDataPtr)
        add     hl,de                           ; HL => row in tile data
        ld      a,(screenX)
        ld d,0 \ ld e,a
        add     hl,de                           ; HL => where in tile data to start drawing this row from
drawLevelNextX:
        push    hl                              ; save pointer to current position in tile data
        call    checkObjectTrigger              ; check if tile is an object
        pop     hl                              ; HL => tile
        push    hl                              ; save it again
        call    checkE2Trigger
        ld h,0 \ ld l,a                         ; HL = tile number
        add     hl,hl
        add     hl,hl
        add     hl,hl
        ld      de,(pTiles)
        add     hl,de                           ; HL => tile sprite
        ex      de,hl                           ; DE => tile sprite
        ld      hl,(dlY)                        ; L = (dlY)
        ld      a,(dlX)
        call    putAlignedSprite                ; draw tile
        pop     hl                              ; restore pointer to current position in tile data
        inc     hl                              ; and increment it
        ld      a,(dlX)
        inc     a                               ; increment x offset / counter
        ld      (dlX),a
        cp      12                              ; have we reached the end of this row?
        jr      c,drawLevelNextX                ; if not, do another tile
        ld      a,(dlY)
        inc     a                               ; increment y offset / counter
        ld      (dlY),a
        cp      8                               ; have we reached the bottom of the screen?
        jr      c,drawLevelNextY                ; if not, do another row
#endif
#ifdef TI84PLUSCE
        ; CE version doesn't draw level here (it draws every frame in the main loop)
        ; but it still needs to scan for objects
        ld      de,(screenY)
        ld      a,(levelWidth)
        ld      d,a
        mlt     de                              ; DE = offset to row in level data
        ld      a,(screenX)
        LDHLA()
        add     hl,de                           ; HL = offset to top-left tile that will be onscreen
        ld      de,(tileDataPtr)
        add     hl,de                           ; HL => tile data
        ld      c,TILE_ROWS
scanLevelRow:
        ld      b,TILE_COLS
        push    hl
scanLevelCol:
        push    bc
        push    hl
        call    checkObjectTrigger              ; check for object
        pop     hl
        pop     bc
        inc     hl                              ; next tile in row
        djnz    scanLevelCol
        pop     de                              ; DE => start of row
        ld      a,(levelWidth)
        LDHLA()
        add     hl,de                           ; HL => next row
        dec     c
        jr      nz,scanLevelRow
#endif

        ; calculate player start position on screen
        ld      a,(startX)                      ; A = start x pos (block value)
        ld      hl,screenX                      ; HL => screen x pos (block value)
        sub     (hl)                            ; A = player x pos on screen (block value)
        add     a,a
        add     a,a
        add     a,a                             ; A = player x pos on screen
        ld      d,a                             ; store it in D
#ifdef TI84PLUSCE
        inc     hl
#endif
        inc     hl
        inc     hl                              ; HL => screen y pos (block value)
        ld      a,(startY)
        sub     (hl)
        add     a,a
        add     a,a
        add     a,a
        ld      e,a                             ; store y pos on screen in e
        ld      ix,p1scry
        ld      (ix),e
        ld      (ix+1),d                        ; save for player 1
        ld      a,6
        add     a,d
        ld      d,a                             ; player 2 starts to the right
        inc     e                               ; and 1 pixel down
        ld      ix,p2scry
        ld      (ix),e
        ld      (ix+1),d                        ; save for player 2

        ; calc screenX & screenY from block values out to pixel values
        ld      hl,(screenX)
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        ld      (screenX),hl
        ld      hl,(screenY)
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        ld      (screenY),hl

        ; calc absolute player coords from startX & startY
        ld      hl,(startX)
        call    setPlayerX
        ld      hl,(startY)
        call    setPlayerY

        ; check if should scroll screen to the right
        ld      a,(p1scrx)                      ; A = player x pos on screen
        cp      48                              ; if player x pos on screen is 48 or more,
        jr      nc,afterDLScrollRight           ; then we're near the right edge of the level, so don't scroll
        ; try to scroll right 4 times
        ld      b,4
drawLevelScrollRight:
        push    bc
        call    scrollRight
        pop     bc
        djnz    drawLevelScrollRight

afterDLScrollRight:
        ; check if should scroll screen down
        ld      a,(p1scry)                      ; A = player y pos on screen
        cp      32                              ; if player y pos on screen is 32 or more,
        jr      nc,afterDLScrollDown            ; then we're near the bottom edge of the level, so don't scroll
        ; try to scroll down 4 times
        ld      b,4
drawLevelScrollDown:
        push    bc
        call    scrollDown
        pop     bc
        djnz    drawLevelScrollDown

afterDLScrollDown:
#ifdef TI84PLUSCE
        INI_FPS_TIMER()
#endif
        ld      hl,miscFlags
        bit     loadingState,(hl)               ; are we loading from a save state?
        jp      nz,mainGameLoop
        res     teleportCmd,(hl)                ; reset this flag in case we just teleported
#ifdef TI84PLUSCE
        call    fadeToGame
#endif
        call    trgLevelStarted
        ld      a,TIP_WALKPAUSE
        ld      d,0                             ; save gbuf in showTip
        bcall(ab5_showTip)                      ; show tip for basic movement, pausing
        jp      mainGameLoop

;------------------------------------------------
; drawLevelResume - update startX & startY to the player position
;   input:  none
;   output: none
;------------------------------------------------
drawLevelResume:
        ld      hl,(p1x)
        inc hl \ inc hl \ inc hl
        HLDIV8()
        ld      a,l
        ld      (startX),a
        ld      hl,(p1y)
        inc hl \ inc hl \ inc hl
        HLDIV8()
        ld      a,l
        ld      (startY),a
        ret

;------------------------------------------------
; getTile - get a tile given the exact coordinates
;   input:  DE = x
;           HL = y
;   output: A = tile number
;           HL => tile in tile data
;           DE = offset to tile
;------------------------------------------------
getTile:
        push    de                              ; save x coord
        HLDIV8()                                ; HL = HL/8 (block y coord)
        ld      a,(levelWidth)
        ld      h,a
        HTIMESL()                               ; HL = offset in tile data to row
        ex      (sp),hl                         ; HL = x coord; offset to row in tile data now on stack
        HLDIV8()                                ; HL = HL/8 (block x coord)
        pop     de                              ; DE = offset to row in tile data
        add     hl,de                           ; HL = offset to tile in tile data
        ex      de,hl
        ld      hl,(tileDataPtr)
        add     hl,de                           ; HL => tile in tile data
        ld      a,(hl)                          ; A = tile number
        ret

;------------------------------------------------
; putTile - put a tile into a level and draw it on the screen
;   input:  HL => tile location in level data
;           A = tile number to put
;   output: HL => tile location in level data
;------------------------------------------------
putTile:
        ld      (hl),a                          ; save new tile number in level data
#ifdef TI83PLUS
        push    hl                              ; save data ptr
        push    af                              ; save tile num
        call    getBlockXYFromHL                ; A = x; L = y; H = 0; ie. HL = y
        add     hl,hl
        add     hl,hl
        add     hl,hl
        ld      de,(screenY)
        or      a                               ; clear carry
        sbc     hl,de                           ; L = y pos to display tile at on screen, H = 0
        ld      c,l                             ; C = y pos to display tile at on screen
        ld      l,a                             ; HL = x coord in level of tile
        add     hl,hl
        add     hl,hl
        add     hl,hl
        ld      de,(screenX)
        or      a
        sbc     hl,de
        ld      b,l                             ; B = x pos to display tile at on screen
        push    bc                              ; save x,y draw pos
        ld      hl,sprWhiteSpace
        call    putClippedMaskedSprite          ; clear the 8x8 position on screen where new tile will be displayed
        pop     bc                              ; restore x,y draw pos
        pop     af                              ; restore tile num to draw
        ld h,0 \ ld l,a
        add     hl,hl
        add     hl,hl
        add     hl,hl
        ld      de,(pTiles)
        add     hl,de                           ; HL => tile sprite
        call    putClippedSprite                ; draw tile sprite
        pop     hl                              ; restore data ptr
#endif
        ret

;------------------------------------------------
; getBlockXYFromHL - get x & y tile coords from a pointer to a tile in level data
;   input:  HL => tile in level data
;   output: A = x (block value)
;           L = y (block value)
;------------------------------------------------
getBlockXYFromHL:
        ld      de,(tileDataPtr)                ; DE => start of level tile data
        or      a                               ; clear carry
        sbc     hl,de                           ; HL = offset to tile in level data
        ld      a,(levelWidth)
        bcall(_divhlbya)                        ; L = dividend (y); A = remainder (x)
        ret

;------------------------------------------------
; getExactXYFromHL - get x & y pixel coords a from a pointer to a tile in level data
;   input:  HL => tile in level data
;   output: DE = x
;           HL = y
;------------------------------------------------
getExactXYFromHL:
        call    getBlockXYFromHL                ; get the block values first
#ifdef TI83PLUS
        ld      h,0                             ; HL = y (block)
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; HL = y (pixel)
        ex      de,hl                           ; DE = y (pixel)
        ld      l,a
        ld      h,0                             ; HL = x (block)
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; HL = x (pixel)
        ex      de,hl                           ; DE = x; HL = y (both pixel coords)
#endif
#ifdef TI84PLUSCE
        ld      e,a
        ld      d,8
        mlt     de                              ; DE = x
        ld      h,8
        mlt     hl                              ; HL = y
#endif
        ret

;------------------------------------------------
; checkWall - check block coords to see if tile is a wall tile
;   input:  B = block x coord
;           C = block y coord
;   output: HL => tile in tile data
;           CA = is a wall tile
;   notes:  if x,y pos is outside bounds of the level, will always return NC
;           x,y pos can only be outside of level by one block
;------------------------------------------------
checkWall:
        ld      a,b
        or      c                               ; A = x & y OR'd together
        bit     7,a                             ; check if either was negative
        jr      nz,checkWallBoundError
        ld      a,(levelWidth)
        cp      b                               ; compare level width to x pos
        jr      z,checkWallBoundError           ; if they're equal, outside level
        ld      a,(levelHeight)
        cp      c                               ; ditto for height & y pos
        jr      z,checkWallBoundError
        ld      hl,(levelWidth-1)               ; H = level width
        ld      l,c                             ; L = block y coord
        push    bc                              ; save block x coord to stack
        HTIMESL()                               ; HL = offset to row in tile data
        pop     bc
        ld de,0 \ ld e,b                        ; DE = block x coord
        add     hl,de                           ; HL = offset to tile in tile data
        ld      de,(tileDataPtr)
        add     hl,de                           ; HL => tile in tile data
        ld      a,(hl)                          ; A = tile number
        cp      TILE_FIRSTWALL                  ; is it higher than the first wall tile?
        ccf                                     ; complement carry
        ret     nc                              ; leave if tile isn't a wall
        cp      TILE_LASTWALL+1                 ; is it higher than the last wall tile?
        ret                                     ; ret regardless with the NC/CA result
checkWallBoundError:
        or      a                               ; clear carry
        ret

;------------------------------------------------
; checkE2Trigger - check if a tile is an enemy2 trigger, and if so, check if it should be represented as a bare floor tile
;   input:  HL => tile in level data
;   output: HL = tile to work with
;           A = tile to work with
;------------------------------------------------
checkE2Trigger:
        ld      a,(hl)                          ; A = tile
        cp      TILE_ENEMYTRIGGER+1             ; check if enemy2 trigger
        jr      nz,checkE2TriggerDone           ; if not, nothing to check
        ld      d,a                             ; save tile type
        inc     hl
        ld      a,(hl)                          ; A = tile next to the trigger
        or      a                               ; check if it's a bare floor tile
        ld      a,d                             ; A = original tile
        jr      nz,checkE2TriggerDone           ; if the next tile isn't a bare floor tile, stick with original tile
        xor     a                               ; otherwise, we'll display the enemy tile as a bare floor tile
checkE2TriggerDone:
#ifdef TI83PLUS
        ld      h,0
#endif
#ifdef TI84PLUSCE
        or  a \ sbc hl,hl
#endif
        ld      l,a                             ; HL = tile to display
        ret

;------------------------------------------------
; openDoor - open a normal door
;   input:  A = closed door tile
;           HL => tile in level data
;   output: none
;------------------------------------------------
openDoor:
        ld      ix,numDoors
        inc     (ix)                            ; inc doors counter for this level
        sub     OPEN_DOOR                       ; A = new tile number for open door
        push    af                              ; save tile number
        call    putTile                         ; put new tile & draw it
        call    getExactXYFromHL                ; DE = x; HL = y (for new animation)
        pop     af                              ; A = open door tile number
        sub     ANIM_DOOR                       ; A = animation type
        ld      b,a                             ; B = animation type
        ld      c,2                             ; C = initial counter value
        jp      newAnim                         ; set up new animation & ret

;------------------------------------------------
; checkAdjFireDoor - check an adjacent tile to see if it's a fire door tile that can be closed
;   input:  HL => tile to check
;   output: Z = tile is an open fire door
;------------------------------------------------
checkAdjFireDoor:
        ld      a,(hl)
        cp      TILE_FIREDOORV
        ret     z
        cp      TILE_FIREDOORH
        ret

;------------------------------------------------
; closeFireDoor - close a fire door
;   input:  DE => tile in level data
;   output: none
;------------------------------------------------
closeFireDoor:
        ex      de,hl                           ; HL => tile in level data
        ld      a,(hl)                          ; A = open fire door tile
closeFireDoorExec:
        add     a,CLOSE_FIRE_DOOR               ; A = closed fire door tile
        call    putTile                         ; put tile & display it
        dec     hl
        call    checkAdjFireDoor
        jr      z,closeFireDoorExec
        inc     hl
        inc     hl
        call    checkAdjFireDoor
        jr      z,closeFireDoorExec
        dec     hl
#ifdef TI83PLUS
        ld      de,(levelWidth)
        ld      d,0                             ; DE = level width, aka offset to tile above or below the current tile
#endif
#ifdef TI84PLUSCE
        ld      a,(levelWidth)
        ld de,0 \ ld e,a
#endif
        or      a                               ; clear carry
        sbc     hl,de                           ; HL => up one tile
        call    checkAdjFireDoor                ; is it an open fire door?
        jr      z,closeFireDoorExec             ; if so, go back and update the tile
        add     hl,de
        add     hl,de                           ; HL => down two tiles
        call    checkAdjFireDoor
        jr      z,closeFireDoorExec
        jp      trgFireDoor

;------------------------------------------------
; countdownShake - check if LCD should be shaking (during countdown sequence)
;   input:  none
;   output: none
;------------------------------------------------
countdownShake:
        ld      a,(shake)
        or      a
        jr      nz,updateShake
        ld      a,(time)
        srl     a
        add     a,40
        ld      b,a
        call    getRandom
        or      a
        ret     nz
        ; fall through to updateShake with A=0 to start a new shake
#ifdef TI83PLUS
updateShake:
        ld      l,a
        srl     l
        ld      h,0                             ; HL = counter/2 (different value every 2nd frame)
        inc     a                               ; inc counter
        and     %00000111                       ; resets counter if it hits 8
        ld      (shake),a
        ld      de,shakeTable
        add     hl,de
        ld      a,(hl)                          ; A = Z-offset to send to LCD driver
        call    $000B                           ; more efficient version of bcall(_lcdbusy)
        out     ($10),a                         ; set Z-offset
#endif
#ifdef TI84PLUSCE
updateShake:
        LDHLA()
        srl     l
        inc     a
        and     %00000111
        ld      (shake),a
        ld      de,shakeTable
        add     hl,de
        ld      a,(hl)
        bit     7,a
        jr      nz,updateShakeScrollDown
updateShakeScrollUp:
        ld      hl,scrollUp
        jr      updateShakeDewIt
updateShakeScrollDown:
        neg
        ld      hl,scrollDown
updateShakeDewIt:
        ld      (__shakeCall),hl
        ld      b,a
updateShakeScrollLoop:
        push    bc
__shakeCall             = $+1
        call    $000000
        pop     bc
        djnz    updateShakeScrollLoop
#endif
        ret

;------------------------------------------------
; showTimer - show countdown timer in custom font
;   input:  none
;   output: none
;------------------------------------------------
showTimer:
#ifdef TI83PLUS
        ld      hl,gbuf+12                      ; HL => second line of gbuf
        ld      ix,timerFrame                   ; IX => data for frame
        ld      de,11                           ; DE = 11 bytes to next gbuf line each loop
        ld      b,11                            ; B = number of lines to clear portion on
showTimerFrame:
        ld      a,(hl)
        and     (ix)                            ; clear bits
        inc     ix
        or      (ix)                            ; write bits
        ld      (hl),a
        inc     hl
        inc     ix
        ld      a,(hl)
        and     (ix)                            ; clear bits
        inc     ix
        or      (ix)                            ; write bits
        ld      (hl),a
        add     hl,de                           ; HL => start of next gbuf line
        inc     ix
        djnz    showTimerFrame                  ; repeat B times
        ld      hl,(time)
        ld      h,0                             ; HL = timer value
        bcall(_divHLby10)                       ; HL = first character to show; A = second
        push    af                              ; save 2nd char
        ld      a,3
        call    showTimerChar                   ; show first char
        pop     af
        ld      h,0
        ld      l,a
        ld      a,8
        ; fall through to showTimerChar
#endif
#ifdef TI84PLUSCE
        ld      a,(time)
        LDHLA()
        ld      a,10
        call    _divHLbyA
        push    af
        ld      e,4
        call    showTimerChar
        pop     af
        ld      l,a
        ld      e,12
        ; fall through to showTimerChar
#endif

;------------------------------------------------
; showTimerChar - show a timer character in custom font
;   input:  HL = number to show (just L for CE version)
;           A = x pos to show it at (83P)
;           E = x pos for drawTransSprite (CE)
;   output: none
;------------------------------------------------
showTimerChar:
#ifdef TI83PLUS
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; HL = offset to sprite
        ld      de,sprTimerNumbers
        add     hl,de                           ; HL => sprite
        ex      de,hl
        ld      l,3                             ; y pos
        jp      putSprite                       ; putSprite and ret
#endif
#ifdef TI84PLUSCE
        ld      a,32
        ld      (__dtsHeight),a
        ld      h,128
        mlt     hl
        add     hl,hl
        add     hl,hl
        ld      bc,sprTimer
        add     hl,bc
        ld      d,4
        call    drawTransSprite
        ld      a,16
        ld      (__dtsHeight),a
        ret
#endif

;------------------------------------------------
; randomExplosion - generate a random on screen explosion
;   input:  none
;   output: none
;------------------------------------------------
randomExplosion:
#ifdef TI83PLUS
        ld      b,96-8
#endif
#ifdef TI84PLUSCE
        ld      b,(TILE_COLS-1)*8
#endif
        call    getRandom                       ; get a random x offset
        ld      hl,(screenX)                    ; HL = x pos of left of screen
#ifdef TI83PLUS
        ld      e,a
        ld      d,0                             ; DE = random x offset
#endif
#ifdef TI84PLUSCE
        ld      de,0
        ld      e,a
#endif
        add     hl,de
        ex      de,hl                           ; DE = x pos for new explosion
#ifdef TI83PLUS
        ld      b,64-8
#endif
#ifdef TI84PLUSCE
        ld      b,(TILE_ROWS-1)*8
#endif
        call    getRandom                       ; get a random y offset
        ld      hl,(screenY)
#ifdef TI83PLUS
        ld      c,a
        ld      b,0
#endif
#ifdef TI84PLUSCE
        ld      bc,0
        ld      c,a
#endif
        add     hl,bc                           ; HL = y pos for new explosion
        ld      bc,0*256+ANIM_EXPLOSION         ; B = animation type; C = counter ini
        jp      newAnim                         ; set up new explosion & ret

;------------------------------------------------
; checkLevelFinished - check if player is in the right location to finish the level
;   input:  HL => player data
;   output: CA = player is in the right spot or player isn't active (in which case it's irrelevant), otherwise NC
;           IX => finish location data [nextLevel,x,y] if player is in a finish location
;------------------------------------------------
checkLevelFinished:
        ld      (pPlayer),hl                    ; save player data pointer
        ld      de,PP
        add     hl,de
        bit     0,(hl)                          ; is this player active?
        jr      z,clfNotActive                  ; if not, no checks to make
        call    loadPlayerCollideObject1
        ld      ix,aFinish
        ld      b,NUM_FINISHES
clfLoop:
        push    bc
        ld      a,(ix)                          ; A = next level for this finish position
        inc     a                               ; was it $FF?
        or      a                               ; clear carry
        jr      z,clfEndLoop                    ; if next level is $FF, then this finish position isn't enabled
        ld      bc,4                            ; offset if large finish
        ld      a,c
        add     a,a                             ; A = width & height value to use
        ld      l,(ix+1)
#ifdef TI83PLUS
        ld      h,0                             ; HL = finish tile x
        add hl,hl \ add hl,hl \ add hl,hl       ; HL = finish x
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        add     hl,bc                           ; HL = centre portion of finish position
        ld      de,collideObject2
        ex      de,hl
        ld      (hl),e                          ; load x LSB
        inc     hl
        ld      (hl),d                          ; load x MSB
        inc     hl
        ld      (hl),a                          ; load width
        inc     hl
        ex      de,hl                           ; DE => collideObject2 y data
        ld      l,(ix+2)
#ifdef TI83PLUS
        ld      h,0                             ; HL = finish tile y
        add hl,hl \ add hl,hl \ add hl,hl       ; HL = finish y
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        add     hl,bc                           ; centre portion of finish position
        ex      de,hl
        ld      (hl),e                          ; load y LSB
        inc     hl
        ld      (hl),d                          ; load y MSL
        inc     hl
        ld      (hl),a                          ; load height
        call    checkCollision                  ; check if player is colliding with centre of finish position
clfEndLoop:
        pop     bc
        ret     c                               ; if player is in a valid correct position, ret
        ld      de,3
        add     ix,de                           ; IX => next finish location
        djnz    clfLoop
        ; if player isn't on a finish position, clear carry and ret
        or      a
        ret
clfNotActive:
        scf                                     ; player not active, so allow level to be finished if other player is in the right position
        ret

;------------------------------------------------
; setPlayerX - set p1x and p2x slightly apart from each other
;   input:  L = tile x coord to use
;   output: none
;------------------------------------------------
setPlayerX:
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        ld      (p1x),hl
        ld      de,6
        add     hl,de
        ld      (p2x),hl
        ret

;------------------------------------------------
; setPlayerY - set p1y and p2y slightly apart from each other
;   input:  L = tile y coord to use
;   output: none
;------------------------------------------------
setPlayerY:
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        ld      (p1y),hl
        inc     hl
        ld      (p2y),hl
        ret

#ifdef TI83PLUS
;------------------------------------------------
; darkMask - draw dark mask overlay for "dark" levels
;   input:  none
;   output: none
;------------------------------------------------
darkMask:
        ; prep maskBuffer
        ld      a,$FF
        ld      (maskBuffer),a
        ld      (maskBuffer+7),a
        ; get ready for main loop
        ld      de,gbuf
        ld      b,64                            ; B = 64 rows to mask
dmMainLoop:
        ; first write the 12-byte overlay for this row
        ld      a,b
        and     %00000011
        jr      nz,dmStartOverlay
        ld      hl,darkOverlay                  ; every 4th row, go back to the start of darkOverlay
dmStartOverlay:
        ld      a,(hl)                          ; A = byte to use for overlay in this row
        push    hl
        push    bc
        ld      hl,overlayBuffer
        ld      b,12                            ; B = 12 bytes to write
dmOverlayLoop:
        ld      (hl),a
        inc     hl
        djnz    dmOverlayLoop
        ; check active players and mask the overlay as necessary
        pop     bc                              ; we need the main loop counter
        push    bc                              ; but also still need it saved for the end of the main loop
        ld      a,(p1p)
        or      a
        ld      ix,p1x
        call    nz,dmPlayerView
        pop     bc
        push    bc
        ld      a,(p2p)
        or      a
        ld      ix,p2x
        call    nz,dmPlayerView
        ; apply the 12-byte over to this row of gbuf
        ld      b,12
        ld      hl,overlayBuffer
dmApplyLoop:
        ld      a,(de)                          ; get byte from gbuf
        or      (hl)                            ; apply overlay
        ld      (de),a                          ; save it
        inc     de
        inc     hl
        djnz    dmApplyLoop
        pop     bc
        pop     hl
        inc     hl                              ; HL => next byte of darkOverlay
        djnz    dmMainLoop
        ret

dmPlayerView:
        ld      a,b
        sub     64
        neg                                     ; A = row (or y coord) that we're working on
        add     a,19                            ; add y offset to where player would be positioned inside the dark overlay
        sub     (ix+PSY)                        ; check it against the player y coord on screen
        ret     c                               ; leave if player is too far down past this row
        cp      46
        ret     nc                              ; leave if player is too far up past this row
        ; if code reached here, then overlayBuffer needs to be combined with a row of darkMaskData
        push    de                              ; save gbuf pointer
        ld      l,a
        ld      h,0                             ; HL = row of darkMaskData to use
        add     hl,hl                           ; x2
        ld      c,l
        ld      b,h
        add     hl,hl                           ; x4
        add     hl,bc                           ; x6
        ld      bc,darkMaskData
        add     hl,bc                           ; HL => row of darkMaskData to use
        ld      de,maskBuffer+1
        push    de
        ld      bc,6
        ldir
        pop     hl
        ld      b,6                             ; B = 6 bytes of overlayBuffer to modify
        ld      a,(ix+PSX)                      ; A = player x coord on screen
        sub     20                              ; A = x coord to start applying darkMaskData
        jr      nc,dmXOffset                    ; if it wasn't negative, we can start calculating the offset
dmXNeg:
        dec     b                               ; one less byte to use for this row
        inc     hl
        add     a,8
        jr      nc,dmXNeg                       ; if still negative, loop again
dmXOffset:
        push    hl
        ld      hl,overlayBuffer
        ADIV8()                                 ; A = offset for overlayBuffer, C = bit shift
        ld      d,0
        ld      e,a
        add     hl,de                           ; HL => first byte of overlayBuffer to modify
        ld      a,c
        or      a                               ; any bit shifting required?
        jr      z,dmXStart
        inc     b                               ; if we'll be shifting, then there is an extra byte of overlayBuffer to modify
dmXStart:
        ld      (tempB),a                       ; save shift counter
dmXLoop:
        ex      (sp),hl                         ; HL => row of darkMaskData to use, (SP) => first byte of overlayBuffer to modify
        ld      a,(tempB)
        or      a                               ; any shifting?
        ld      e,(hl)                          ; A = byte of darkMaskData
        jr      z,dmXAfterShift
        dec     hl
        ld      d,(hl)
        inc     hl
dmXShift:
        srl     d
        rr      e
        dec     a
        jr      nz,dmXShift
dmXAfterShift:
        inc     hl                              ; HL => next byte of darkMaskData
        ex      (sp),hl                         ; HL => byte of overlayBuffer, (SP) => byte of darkMaskData
        ld      a,(hl)
        and     e
        ld      (hl),a
        inc     hl                              ; HL => next byte of overlayBuffer
        djnz    dmXLoop
        pop     hl                              ; clear stack junk
        pop     de                              ; restore gbuf pointer
        ret
#endif

.end
