;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Level routines (TI-84+CE only)                                ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; modifyText - parse text to adjust for the larger screen (such as mission objectives)
;   input:  HL => text data
;   output: none
;------------------------------------------------
modifyText:
        ; fix row (y pos)
        ld      a,(hl)
        add     a,a
        add     a,40
        ld      (hl),a
        inc     hl
        ; fix col (x pos)
        ld      a,(hl)
        add     a,24
        ld      (hl),a
        inc     hl
        ; scan to next line (or end of text)
mtFindNextLine:
        ld      a,(hl)
        inc     hl
        or      a
        jr      z,modifyText
        inc     a
        ret     z
        jr      mtFindNextLine

;------------------------------------------------
; checkE2TriggerIY - check if a tile is an enemy2 trigger, and if so, check if it should be represented as a bare floor tile
;   input:  IY => tile in level data
;   output: IY => tile in level data
;           L = tile to work with
;------------------------------------------------
checkE2TriggerIY:
        ld      a,(iy)                          ; A = tile
        cp      TILE_ENEMYTRIGGER+1             ; check if enemy2 trigger
        ld      l,a
        ret     nz                              ; if not, nothing to check
        ld      a,(iy+1)                        ; A = tile next to the trigger
        or      a                               ; check if it's a bare floor tile
        ret     nz                              ; if the next tile isn't a bare floor tile, stick with original tile
        ld      l,0
        ret

;------------------------------------------------
; getEpLvlOffset - get offset for tables based off episode and level numbers
;   input:  none
;   output: HL = offset
;------------------------------------------------
getEpLvlOffset:
        ld      a,(episode)
        LDHLA()
        ld      de,episodeTable
        add     hl,de
        ld      l,(hl)
        ld      h,3
        mlt     hl
        ld      a,(level)
        ld      e,a
        ld      d,3
        mlt     de
        add     hl,de
        ret

.end
