;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Menu routines                                                 ;
;                                                               ;
;===============================================================;

;;;menuNum
;;;0 main menu
;;;1 episode menu
;;;2 options 1
;;;3 options 2
;;;4 view high scores
;;;5 enter passcode
;;;6 enter high score name

;------------------------------------------------
; mainMenu - start the main menu
;   input:  none
;   output: none
;------------------------------------------------
mainMenu:
        xor     a
        ld      (menuNum),a

        call    iniMenu

continueMenu:
        xor     a
        ld      (menuItem),a
        ld      (frame),a

menuLoop:
        call    doMenuBackground                ; do stars and logo stuff

        ; load splash
#ifdef TI83PLUS
        ld      a,(frame)
        bit     0,a
        call    z,loadMenuSplash
#endif
#ifdef TI84PLUSCE
        call    loadMenuSplash
#endif

        ; write menu text depending on which menu we're on
        ld      hl,(menuNum)
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        ld      de,menuStrTable
        add     hl,de
        LDHLIND()
        push hl \ pop ix
        set     textInverse,(iy+textFlags)
#endif
#ifdef TI84PLUSCE
        ld      h,3
        mlt     hl
        ld      de,menuStrTable
        add     hl,de
        ld      ix,(hl)                         ; IX => string data
#endif
        ld      b,(ix)                          ; B = number of strings to write
        ld      a,(menuNum)
        ld      c,a
        xor     a                               ; used to determine which string should flash
        inc     ix
        ld      hl,menuItem
menuStrLoop:
        push    bc
        push    hl
        push    af
        cp      (hl)                             ; is this string the selected menu item?
#ifdef TI83PLUS
        ld      d,(ix)
        ld      e,(ix+1)
        push ix \ pop hl
        inc hl \ inc hl
        jr      nz,menuShowStr
        ld      a,c
        cp      4
        jr      nc,menuShowStr
        ld      a,(frame)
        bit     3,a
        jr      z,menuShowStr
        call    skipString
        jr      menuAfterStr
menuShowStr:
        call    vPuts
menuAfterStr:
        push hl \ pop ix
#endif
#ifdef TI84PLUSCE
        ld      a,COLOUR_WHITE
        jr      nz,msSetColour
        ld      a,c
        cp      4
        ld      a,COLOUR_WHITE
        jr      nc,msSetColour
        ld      a,$FF
msSetColour:
        ld      (__psColour),a                  ; set colour
        ld      l,(ix)                          ; L = y
        inc     ix
        ld      e,(ix)
        ld      d,2
        mlt     de                              ; DE = x
        inc     ix                              ; IX => string ascii
        call    printString
#endif
        pop     af
        inc     a
        pop     hl
        pop     bc
        djnz    menuStrLoop
        
        ; check if episode logos need to be shown (Episode menu)
        ld      a,(menuNum)
        dec     a                               ; menu 1?
        push    af                              ; save for next check
        call    z,showEpisodesMenu
        pop     af

        ; check if additional text needs to be shown (Options Pg.1 menu)
        dec     a                               ; menu 2?
        call    z,showOptionsMenu

        ld      a,(menuNum)
        sub     4
        jr      c,menuStdKeys
        ld      l,a
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,3
        mlt     hl
#endif
        ld      de,menuJumpTable
        add     hl,de
        LDHLIND()
        jp      (hl)

menuStdKeys:
#ifdef TI83PLUS
        res     textInverse,(iy+textFlags)
        bcall(_grbufcpy)
#endif
#ifdef TI84PLUSCE
        call    vramFlip
#endif
        bcall(_getcsc)
        cp      GK_2ND
        jr      z,menuSelect
        cp      GK_ENTER
        jr      z,menuSelect
        cp      GK_MODE
        jr      z,menuSelect
        cp      GK_CLEAR
        jp      z,quit
        cp      GK_UP
        jr      z,menuUp
        cp      GK_DOWN
        jr      z,menuDown
        
        jp      menuLoop

menuUp:
        ld      hl,menuItem                     ; HL => menuItem
        dec     (hl)
        bit     7,(hl)
        jp      z,menuLoop                      ; if it didn't go to -1, no correction needed
        ex      de,hl                           ; DE => menuItem
        ld      a,(menuNum)
        LDHLA()
        ld      bc,menuNumTable
        add     hl,bc
        ld      a,(hl)                          ; A = # options in this menu
        dec     a                               ; A = last option (0..max)
        ld      (de),a                          ; set it
        jp      menuLoop

menuDown:
        ld      hl,menuItem
        inc     (hl)
        ex      de,hl                           ; DE => menuItem
        ld      a,(menuNum)
        LDHLA()
        ld      bc,menuNumTable
        add     hl,bc                           ; HL => # options in this menu
        ex      de,hl
        ld      a,(de)
        cp      (hl)
        jp      nz,menuLoop                     ; if it didn't go to 3, no correction needed
        ld      (hl),0                          ; otherwise, go to top selection
        jp      menuLoop

menuSelect:
        ld      a,(menuNum)
        or      a
        jr      z,menuMainSelect
        dec     a
        jr      z,menuEpisodeSelect
        dec     a
        jr      z,menuOptions1Select
        dec     a
        jr      z,menuOptions2Select
        jp      menuLoop

menuViewHighScoresSelect:
        jr      doMenuMain

menuOptions2Select:
        ld      a,(menuItem)
        or      a
        jp      z,confirmResetAchv              ; confirm to reset achievements
        dec     a
        jp      z,confirmResetTips              ; confirm to reset in-game tips
        dec     a
        jp      z,confirmDisableTips            ; confirm to disable in-game tips
        dec     a
        jp      z,confirmResetHigh              ; confirm to reset high scores
        dec     a
        jp      z,menuIntro
        jr      doMenuMain

menuOptions1Select:
        ld      a,(menuItem)
        ld      b,OPTION_DIFFICULTY
        or      a
        jr      z,toggleOption                  ; toggle difficulty
        sla     b
        dec     a
        jr      z,toggleOption                  ; toggle players
        sla     b
        dec     a
        jr      z,toggleOption                  ; toggle retreat firing
        sla     b
        dec     a
        jr      z,toggleOption                  ; toggle smooth walls
        dec     a
        jr      z,doMenuOptions2
        jr      doMenuMain

toggleOption:
        ld      hl,(pOptions)
        ld      a,(hl)                          ; load options flags
        xor     b                               ; toggle one of them
        ld      (hl),a                          ; save them again
        jp      menuLoop

menuEpisodeSelect:
        ld      a,(menuItem)
        ld      b,a                             ; B = episode # (except if returning to main menu)
        cp      NUMEPISODES
        jp      c,newGame
        jr      doMenuMain

menuMainSelect:
        ld      a,(menuItem)
        or      a
        jr      z,doMenuEpisode
        dec     a
        jr      z,doMenuPasscode
        dec     a
        jr      z,doMenuOptions1
        dec     a
        jr      z,doMenuViewHighScores
        dec     a
        jr      z,doMenuViewAchievements
        jp      quit

doMenuMain:
        xor     a
        ld      (menuNum),a
        jp      continueMenu

doMenuEpisode:
        ld      a,1
        ld      (menuNum),a
        jp      continueMenu

doMenuPasscode:
        xor     a
        ld      (charCount),a
        ld      hl,passcode
        ld      (pString),hl
        ld      a,5
        ld      (menuNum),a
        jp      continueMenu

doMenuOptions1:
        ld      a,2
        ld      (menuNum),a
        jp      continueMenu

doMenuOptions2:
        ld      a,3
        ld      (menuNum),a
        jp      continueMenu

doMenuViewHighScores:
        ld      a,4
        ld      (menuNum),a
        jp      continueMenu

doMenuViewAchievements:
        bcall(ab5_viewAchievements)
        jp      mainMenu

;------------------------------------------------
; starClipSub - modify coord in C for bottom/right clipping
;   input:  C = coord to modify
;   output: C = updated coord
;------------------------------------------------
starClipSub:
        ld      a,-STAR_CLIP
        add     a,c
        ld      c,a
        ret

;------------------------------------------------
; starClipAdd - modify coord in C for top/left clipping
;   input:  C = coord to modify
;   output: C = updated coord
;------------------------------------------------
starClipAdd:
        ld      a,STAR_CLIP
        add     a,c
        ld      c,a
        ret

;------------------------------------------------
; showEpisodesMenu - show the episode logos
;   input:  none
;   output: none
;------------------------------------------------
showEpisodesMenu:
#ifdef TI83PLUS
        ld      b,2
        call    delay                           ; the episode menu needs a little slowing down
        ld      c,NUMEPISODES                   ; C = # logos to show
        ld      ix,ep1logo                      ; IX => start of logo data
        ld      hl,gbuf+(18*12)                 ; HL => where to draw
showEpisodeLogo:
        ld      b,60                            ; B = 60 bytes to draw (5 rows)
        push    ix
        push    hl
        ld      a,(menuItem)
        sub     NUMEPISODES
        neg
        cp      c
        jr      nz,showEpisodeLogoLoop
        ld      a,(frame)
        bit     3,a
        jr      nz,endShowEpisodeLogo           ; if this is the selected episode, it blinks
showEpisodeLogoLoop:
        ld      a,(ix)
        and     (hl)
        ld      (hl),a
        inc     ix
        inc     hl
        djnz    showEpisodeLogoLoop
endShowEpisodeLogo:
        pop     hl
        ld      de,60+24
        add     hl,de
        pop     ix
        ld      de,60
        add     ix,de
        dec     c
        jr      nz,showEpisodeLogo
#endif
#ifdef TI84PLUSCE
        ld      b,NUMEPISODES
        ld      ix,(pVbuf)
        ld      de,320*101+142
        add     ix,de
showEpisodeLogo:
        ld      a,(menuItem)
        sub     NUMEPISODES
        neg
        cp      b
        ld      a,COLOUR_WHITE
        jr      nz,showEpisodeLogoSetColour
        ld      a,$FF
showEpisodeLogoSetColour:
        ld      (__showEpisodeColour),a
        push    bc
        push    ix
        ld      hl,(pDatFile)
        ld      de,logos
        add     hl,de
        ld      de,logoBuffer
        ld      ix,huffTree
        ld      a,NUMEPISODES
        sub     b
        ld      b,a
        call    HuffExtr
        pop     ix                              ; IX => vbuf ptr
        ld      hl,logoBuffer                   ; HL => unpacked logo
        ld      c,10                            ; C = 10 rows
showEpisodeLogoRow:
        ld      b,13                            ; B = 13 bytes per row
__showEpisodeColour     = $+1
        ld      de,128*256+COLOUR_WHITE         ; D = bit mask, E = colour
showEpisodeLogoPixel:
        ld      a,(hl)
        and     d
        jr      z,showEpisodeLogoNextPixel
        ld      (ix),e
showEpisodeLogoNextPixel:
        inc     ix
        rrc     d
        jr      nc,showEpisodeLogoPixel
        inc     hl
        djnz    showEpisodeLogoPixel
        ld      de,320-104
        add     ix,de                           ; IX => vbuf ptr to next row
        dec     c
        jr      nz,showEpisodeLogoRow
        ld      de,320*5
        add     ix,de
        pop     bc
        djnz    showEpisodeLogo
#endif
        ret

;------------------------------------------------
; showOptionsMenu - show the options for the first Options menu
;   input:  none
;   output: none
;------------------------------------------------
showOptionsMenu:
        ; show difficulty
#ifdef TI83PLUS
        ld      hl,strNormal
#endif
#ifdef TI84PLUSCE
        ld      ix,strNormal
#endif
        ld      de,(pOptions)
        ld      a,(de)
        and     OPTION_DIFFICULTY               ; normal difficulty?
        jr      z,showDifficulty
#ifdef TI83PLUS
        ld      hl,strTough
#endif
#ifdef TI84PLUSCE
        ld      ix,strTough
#endif
showDifficulty:
#ifdef TI83PLUS
        ld      de,17*256+66
        call    vPuts
#endif
#ifdef TI84PLUSCE
        ld      a,(menuItem)
        ld      b,$FF
        or      a
        jr      z,showDifficultyC
        ld      b,COLOUR_WHITE
showDifficultyC:
        ld      a,b
        ld      de,200
        ld      l,100
        call    printStringC
#endif

        ; show players
#ifdef TI83PLUS
        ld      hl,strOne
#endif
#ifdef TI84PLUSCE
        ld      ix,strOne
#endif
        ld      de,(pOptions)
        ld      a,(de)
        and     OPTION_PLAYERS
        jr      z,showPlayers
#ifdef TI83PLUS
        ld      hl,strTwo
#endif
#ifdef TI84PLUSCE
        ld      ix,strTwo
#endif
showPlayers:
#ifdef TI83PLUS
        ld      de,24*256+66
        call    vPuts
#endif
#ifdef TI84PLUSCE
        ld      a,(menuItem)
        ld      b,$FF
        dec     a
        jr      z,showPlayersC
        ld      b,COLOUR_WHITE
showPlayersC:
        ld      a,b
        ld      de,224
        ld      l,115
        call    printStringC
#endif

        ; show retreat firing
#ifdef TI83PLUS
        ld      hl,strOff
#endif
#ifdef TI84PLUSCE
        ld      ix,strOff
#endif
        ld      de,(pOptions)
        ld      a,(de)
        and     OPTION_RETREAT
        jr      z,showRetreat
#ifdef TI83PLUS
        ld      hl,strOn
#endif
#ifdef TI84PLUSCE
        ld      ix,strOn
#endif
showRetreat:
#ifdef TI83PLUS
        ld      de,31*256+66
        call    vPuts
#endif
#ifdef TI84PLUSCE
        ld      a,(menuItem)
        ld      b,$FF
        cp      2
        jr      z,showRetreatC
        ld      b,COLOUR_WHITE
showRetreatC:
        ld      a,b
        ld      de,224
        ld      l,130
        call    printStringC
#endif

        ; show smooth walls
#ifdef TI83PLUS
        ld      hl,strOff
#endif
#ifdef TI84PLUSCE
        ld      ix,strOff
#endif
        ld      de,(pOptions)
        ld      a,(de)
        and     OPTION_WALLS
        jr      z,showWalls
#ifdef TI83PLUS
        ld      hl,strOn
#endif
#ifdef TI84PLUSCE
        ld      ix,strOn
#endif
showWalls:
#ifdef TI83PLUS
        ld      de,38*256+66
        call    vPuts
#endif
#ifdef TI84PLUSCE
        ld      a,(menuItem)
        ld      b,$FF
        cp      3
        jr      z,showWallsC
        ld      b,COLOUR_WHITE
showWallsC:
        ld      a,b
        ld      de,224
        ld      l,145
        call    printStringC
#endif

        ret

;------------------------------------------------
; showHighScores - show the high score table
;   input:  none
;   output: none
;------------------------------------------------
showHighScores:
#ifdef TI83PLUS
        ld      hl,(pHscores)                   ; HL => high score data in the appvar to display
        call    showStrings
        ; at this point, HL => high score values
        ld      d,21                            ; initial penrow
        ld      b,3                             ; B = 3 high scores to show
shsLoop:
        ld      e,63                            ; E = pencol
        ld      (pencol),de                     ; set pencol
        push    bc                              ; save counter
        push    hl                              ; save data ptr
        LDHLIND()                               ; HL = score to show
        call    showHL                          ; display it
        ld      a,(penrow)
        add     a,6
        ld      d,a                             ; D = new penrow
        pop     hl
        inc     hl
        inc     hl                              ; HL => next high score
        pop     bc
        djnz    shsLoop
        res     textInverse,(iy+textFlags)
        bcall(_grbufcpy)
#endif
#ifdef TI84PLUSCE
        ld      ix,(pHscores)
        call    printBlock
        push ix \ pop hl
        ; at this point, HL => high score values
        ld      bc,3*256+90
shsLoop:
        ld      de,200
        push    bc
        push    hl
        ld      hl,(hl)
        call    _setHLUto0
        call    showHL
        pop     hl
        inc     hl
        inc     hl
        pop     bc
        ld      a,15
        add     a,c
        ld      c,a
        djnz    shsLoop
        call    vramFlip
#endif
        bcall(_getcsc)
        or      a
        jp      z,menuLoop
        jp      doMenuMain

;------------------------------------------------
; enterPasscode - get a passcode from the player
;   input:  none
;   output: none
;------------------------------------------------
enterPasscode:
        call    getString
        jp      nc,menuLoop
#ifdef TI84PLUSCE
        inc     a
        jp      z,viewPasscodes
#endif
        jp      loadFromPasscode

;------------------------------------------------
; enterNewHighScore - get the player's name for the high score table
;   input:  none
;   output: none
;------------------------------------------------
enterNewHighScore:
#ifdef TI83PLUS
        ld      hl,(pPlayer)
        call    showStrings
#endif
#ifdef TI84PLUSCE
        ld      ix,(pPlayer)
        call    printBlock
#endif
        call    getString
        jp      nc,menuLoop
        ret

;------------------------------------------------
; getString - get input to build a string of characters from the player (used in enterPasscode and newHighScore)
;   input:  (pString) => where to store string
;   output: CA = set if done
;------------------------------------------------
getString:
        ld      a,(charCount)
#ifdef TI83PLUS
        ld      hl,35*256+28
        ld      (pencol),hl
#endif
#ifdef TI84PLUSCE
        ld      de,120
#endif
        push    af
        or      a
        jr      z,gsShowBlanks
        ; show characters that have been received so far
        ld      b,a
        ld      hl,(pString)
gsShowCharLoop:
#ifdef TI83PLUS
        push    bc
        push    hl
        ld      a,(hl)
        bcall(_vputmap)
        pop     hl
        inc     hl
        pop     bc
        djnz    gsShowCharLoop
#endif
#ifdef TI84PLUSCE
        push    bc
        push    hl
        ld      a,(hl)
        ld      ix,strPasscodeChar
        ld      (ix),a
        ld      l,130
        call    printString
        pop     hl
        inc     hl
        pop     bc
        djnz    gsShowCharLoop
#endif
gsShowBlanks:
        pop     af
        sub     10
        neg
        or      a
        jr      z,gsDone
        ; show underscore chars to fill out string
        ld      b,a
gsShowBlanksLoop:
#ifdef TI83PLUS
        push    bc
        ld      a,'_'
        bcall(_vputmap)
        pop     bc
        djnz    gsShowBlanksLoop
#endif
#ifdef TI84PLUSCE
        push    bc
        ld      ix,strPasscodeChar
        ld      (ix),'_'
        ld      l,130
        call    printString
        pop     bc
        djnz    gsShowBlanksLoop
#endif

#ifdef TI83PLUS
        res     textInverse,(iy+textFlags)
        bcall(_grbufcpy)
#endif
#ifdef TI84PLUSCE
        call    vramFlip
#endif
        ; check key presses
        bcall(_getcsc)
        or      a
        ret     z
        cp      GK_DEL
        jr      z,gsBackspace
        cp      GK_ENTER
        jr      z,gsEnter
#ifdef TI84PLUSCE
        cp      GK_STAT
        jr      z,gsStat
#endif
        dec     a
        LDHLA()
        ld      de,charTable
        add     hl,de
        ld      a,(hl)
        cp      '.'
        ret     z
        ld      c,a
        ld      de,(pString)
        ld      a,(charCount)
        LDHLA()
        inc     a
        ld      (charCount),a
        add     hl,de
        ld      (hl),c
        ret
gsDone:
        scf
        ret
gsBackspace:
        ld      a,(charCount)
        or      a
        ret     z
        dec     a
        ld      (charCount),a
        ret
#ifdef TI84PLUSCE
gsStat:
        ld      a,$FF
        scf
        ret
#endif
gsEnter:
        ld      a,(charCount)
        LDHLA()
        ld      de,(pString)
        add     hl,de
        sub     10
        neg
        ld      b,a
gsFillLoop:
        ld      (hl),' '
        inc     hl
        djnz    gsFillLoop
        scf
        ret



confirmResetAchv:
        ld      hl,strResetAchv                 ; HL => info box title
        ld      de,strAreYouSure                ; DE => info box text
        ld      a,1                             ; info box is a prompt
        call    showInfoBox
        jp      nc,menuLoop                     ; if "No" selected, return to menu
        ld      hl,(pAchFlags)                  ; HL => data to be filled
        ld      b,ACH_DATA_SIZE                 ; B = # bytes to fill
        xor     a                               ; A = $00 (clearing bytes)
        jr      dataFill

confirmResetTips:
        ld      hl,strResetTips
        ld      de,strAreYouSure
        ld      a,1
        call    showInfoBox
        jp      nc,menuLoop
        xor     a
        jr      resetOrDisableTips

confirmDisableTips:
        ld      hl,strDisableTips
        ld      de,strAreYouSure
        ld      a,1
        call    showInfoBox
        jp      nc,menuLoop
        ld      a,$FF
        ; fall through to resetOrDisableTips

resetOrDisableTips:
        ld      hl,(pTips)
        ld      b,TIPS_DATA_SIZE
        ; fall through to dataFill

;------------------------------------------------
; dataFill - fill up to 255 bytes with a value
;   input:  HL => where to start filling
;           B = number of bytes to fill
;           A = value to fill with
;   output: HL => next byte after data block
;------------------------------------------------
dataFill:
        ld      (hl),a                          ; fill byte
        inc     hl
        djnz    dataFill
        jp      mainMenu

confirmResetHigh:
        ld      hl,strResetHigh
        ld      de,strAreYouSure
        ld      a,1
        call    showInfoBox
        jp      nc,menuLoop
        ld      hl,highScoreNames               ; HL => default high score data
        ld      de,(pHscores)                   ; DE => high score data in appvar
        ld      bc,HSCORE_DATA_SIZE
        ldir                                    ; copy default data
        jp      mainMenu

menuIntro:
        push    af
        jp      doIntro

;------------------------------------------------
; iniMenu - initialise the menu screen
;   input:  none
;   output: none
;------------------------------------------------
iniMenu:
        xor     a
        ld      (__starXOffset),a
        ld      (framexy),a
        ld      a,144
        ld      (framez),a
#ifdef TI84PLUSCE
        ; decompress title image
        ld      hl,(pDatFile)
        ld      de,ablogoce
        add     hl,de
        ld      de,sbuf
        ld      ix,huffTree
        call    HuffExtr
#endif
        ; initialise stars
        ld      bc,NUM_STARS*256+1
        ld      ix,starTable
iniStarsLoop:
        push    bc
        ld      b,2*STAR_MAX_COORD+1
        bcall(ab5_getRandom)
        sub     40
        jr      nz,starIniXOK
        inc     a
starIniXOK:
        ld      (ix),a
        ld      b,2*STAR_MAX_COORD+1
        bcall(ab5_getRandom)
        sub     40
        jr      nz,starIniYOK
        inc     a
starIniYOK:
        ld      (ix+1),a
        pop     bc
        ld      (ix+2),c
        ld      a,1
        add     a,c
        ld      c,a
        ld      de,3
        add     ix,de
        djnz    iniStarsLoop
        ret

;------------------------------------------------
; doMenuBackground - do the stars and logo
;   input:  none
;   output: none
;------------------------------------------------
doMenuBackground:
        ; update (framexy) and (framez) for star field
        ld      hl,framexy
        inc     (hl)
        jr      nc,menuSkipOverflow
        ld      a,(__starXOffset)
        xor     $10
        ld      (__starXOffset),a
menuSkipOverflow:
        bit     0,(hl)
        jr      nz,menuSkipFrameZ
        inc     hl
        inc     (hl)                            ; increment (framez)
menuSkipFrameZ:
        ; update (frame) for text & logo effects
        ld      a,(frame)
#ifdef TI84PLUSCE
        push    af
        bit     0,a
        ld      hl,palMenu
        jr      z,menuPalette
        ld      hl,palMenuAlt
menuPalette:
        ld      bc,PALMENU_SIZE
        ld      de,mpLcdPalette+(32*2)
        ldir
        ; update flash colour
        pop     af
#endif
        inc     a
        and     %00001111
        ld      (frame),a
#ifdef TI84PLUSCE
        call    updateCursorFlash
#endif

#ifdef TI83PLUS
        ld      hl,gbuf
        ld      de,gbuf+1
        ld      (hl),$FF
        ld      bc,768-1
        ldir
#endif
#ifdef TI84PLUSCE
        call    vbufClear
#endif
        
        ; set x,y,z movement values for this frame
        ld      a,(framexy)
        ld      b,a
#ifdef TI83PLUS
        ld      hl,__starXOffset
        add     a,(hl)
#endif
#ifdef TI84PLUSCE
__starXOffset           = $+1
        add     a,$00
#endif
        LDHLA()
        ld      de,starMoveTable
        add     hl,de
        ld      a,(hl)                          ; A = x movement value
        ld      (__starXMove),a
        ld      a,b
        add     a,32
        LDHLA()
        ld      de,starMoveTable
        add     hl,de
        ld      a,(hl)                          ; A = y movement value
        ld      (__starYMove),a
        ld      a,(framez)
        LDHLA()
        ld      de,starMoveTable
        add     hl,de
        ld      a,(hl)                          ; A = z movement value
        srl     a
        ld      (__starZMove),a

        ; move stars and draw them
        ld      b,NUM_STARS
        ld      ix,starTable
starLoop:
        push    bc
        ; move x
        ld      a,(ix)
#ifdef TI83PLUS
        ld      hl,__starXMove
        add     a,(hl)
#endif
#ifdef TI84PLUSCE
__starXMove             = $+1
        add     a,$00
#endif
        ld      c,a
        add     a,-STAR_MAX_COORD
        bit     7,a
        call    z,starClipSub
        ld      a,c
        add     a,STAR_MAX_COORD
        bit     7,a
        call    nz,starClipAdd
        ld      (ix),c                          ; save updated x
        ; move y
        ld      a,(ix+1)
#ifdef TI83PLUS
        ld      hl,__starYMove
        add     a,(hl)
#endif
#ifdef TI84PLUSCE
__starYMove             = $+1
        add     a,$00
#endif
        ld      c,a
        add     a,-STAR_MAX_COORD
        bit     7,a
        call    z,starClipSub
        ld      a,c
        add     a,STAR_MAX_COORD
        bit     7,a
        call    nz,starClipAdd
        ld      (ix+1),c                        ; save updated y
        ; move z
        ld      a,(ix+2)
#ifdef TI83PLUS
        ld      hl,__starZMove
        add     a,(hl)
#endif
#ifdef TI84PLUSCE
__starZMove             = $+1
        add     a,$00
#endif
        and     %00111111
        ld      (ix+2),a
        ; calculate 2D coordinates
        ; calc x
        ld      a,(ix)
        LDHLA()
        bit     7,a
        jr      z,starXCalc
        ld      hl,-1
        ld      l,a
starXCalc:
        STAR_MUL()
        ld      c,(ix+2)
        inc     c
        call    divHLbyCs
#ifdef TI83PLUS
        ld      de,96/2
#endif
#ifdef TI84PLUSCE
        ld      de,320/2
#endif
        add     hl,de
        ; check that it's onscreen
        bit     7,h
        jr      nz,starNext                     ; off left edge, so can't draw
        push    hl
#ifdef TI83PLUS
        ld      de,-96
#endif
#ifdef TI84PLUSCE
        ld      de,-320
#endif
        add     hl,de
        bit     7,h
        pop     hl
        jr      z,starNext                      ; off right edge, so can't draw
        push    hl
        ; calc y
        ld      a,(ix+1)
        LDHLA()
        bit     7,a
        jr      z,starYCalc
        ld      hl,-1
        ld      l,a
starYCalc:
        STAR_MUL()
        ld      c,(ix+2)
        inc     c
        call    divHLbyCs
#ifdef TI83PLUS
        ld      de,64/2
#endif
#ifdef TI84PLUSCE
        ld      de,240/2
#endif
        add     hl,de
        ; check that it's onscreen
        pop     bc                              ; BC = x, clear stack in case we jump to starNext
        bit     7,h
        jr      nz,starNext
        push    hl
#ifdef TI83PLUS
        ld      de,-64
#endif
#ifdef TI84PLUSCE
        ld      de,-240
#endif
        add     hl,de
        bit     7,h
        pop     hl
        jr      z,starNext
        ; BC = x, L = y
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        ld d,h \ ld e,l
        add     hl,hl
        add     hl,de
        srl c \ rr b
        srl c \ rr b
        srl c \ rr b
        rlc b \ rlc b \ rlc b
        ld      a,%01111111
starRotate:
        rrca
        djnz    starRotate
        add     hl,bc
        ld      de,gbuf
        add     hl,de
        and     (hl)
        ld      (hl),a
#endif
#ifdef TI84PLUSCE
        ld      h,160
        mlt     hl
        add     hl,hl
        add     hl,bc
        ld      de,(pVbuf)
        add     hl,de
        ld      a,(ix+2)
        srl     a
        srl     a
        sub     16
        neg
        add     a,16                            ; palette slots 16-31 are grayscale
        ld      (hl),a
#endif
starNext:
        ld      de,3
        add     ix,de
        pop     bc
        dec     b
        jp      nz,starLoop
        ret
        
;------------------------------------------------
; loadMenuSplash - load menu splash screen
;   input:  none
;   output: none
;------------------------------------------------
loadMenuSplash:
#ifdef TI83PLUS
        ld      hl,ablogo
        ld      de,gbuf+(4*12)+1
        ld      a,9
lmsLoop:
        ld      bc,10
        ldir
        inc de \ inc de
        dec     a
        jr      nz,lmsLoop
        ret
#endif
#ifdef TI84PLUSCE
        ld      de,36
        ld      l,32
        ld      bc,124*256+32
        jp      drawImage
#endif

.end
