;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Miscellaneous routines                                        ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; copySprite - copy 8 bytes of data from (HL) to sprBuffer
;   input:  HL => sprite
;   output: DE => sprite in ram
;------------------------------------------------
copySprite:
        ld      de,sprBuffer
        ld      bc,8
        call    copyBCBytes
        ex      de,hl
        ret

;------------------------------------------------
; copy256Bytes - copy 256 bytes of data from (HL) to achBuffer
;   input:  HL => source
;   output: HL => achBuffer
;------------------------------------------------
copy256Bytes:
        ld      de,achBuffer
        ld      bc,256
        ; fall through to copyBCBytes

;------------------------------------------------
; copyBCBytes - copy BC bytes of data from (HL) to (DE)
;   input:  HL => source
;           DE => destination
;           BC = size
;   output: HL => destination
;------------------------------------------------
copyBCBytes:
        push    de
        ldir
        pop     hl
        ret

;------------------------------------------------
; ldAHLind - load 3 bytes at (HL) into AHL
;   input:  HL => 3 bytes to load
;   output: AHL = 24 bit value
;   notes:  preserves all other registers
;------------------------------------------------
ldAHLind:
        push    de
        ld      e,(hl)
        inc     hl
        ld      d,(hl)
        inc     hl
        ld      a,(hl)
        ex      de,hl
        pop     de
        ret

;------------------------------------------------
; ldBDEtoHL - load BDE to (HL)
;   input:  BDE = 24 bit value
;           HL => memory location to save to
;   output: none
;------------------------------------------------
ldBDEtoHL:
        ld      (hl),e
        inc     hl
        ld      (hl),d
        inc     hl
        ld      (hl),b
        ret

;------------------------------------------------
; cpAHLBDE - compare AHL to BDE
;   input:  AHL = first value
;           BDE = second value
;   output: 
;------------------------------------------------
cpAHLBDE:
        cp      b                               ; compare high byte
        ret     nz                              ; if not equal, then answer is already determined
        CPHLDE()                                ; compare low word
        ret

;------------------------------------------------
; divAHLby10 - divide AHL by 10 and store the remainder in C
;   input:  AHL = 24 bit value
;   output: AHL = quotient
;           C = remainder
;   credit: routine originally posted by AssemblyBandit on Omnimaga, optimised by Xeda and modified slightly to suit
;------------------------------------------------
divAHLby10:
        push    de
        push    bc
        ld      d,a
        ld      bc,$180A
        sub     a
dAHLloop1:
        add     hl,hl
        rl      d
        rla
        cp      c
        jr      c,dAHLloop2
        sub     c
        inc     l
dAHLloop2:
        djnz    dAHLloop1
        pop     bc
        ld      c,a
        ld      a,d
        pop     de
        ret

;------------------------------------------------
; showAHL - show AHL in small font at cursor position (left aligned)
;   input:  AHL = 24 bit value
;   output: none
;------------------------------------------------
showAHL:
        ld      de,showHLBuffer+8               ; DE => end of buffer
        ld      b,8                             ; B = number of characters to calculate
        ld      c,a
        xor     a
        ld      (de),a                          ; set NULL terminator at end
        ld      a,c
showAHLMake:
        dec     de
        call    divAHLby10                      ; C = remainder
        push    af
        ld      a,c
        add     a,'0'
        ld      (de),a
        pop     af
        djnz    showAHLMake
        ex      de,hl
; skip all leading zeros
        ld      b,7
showAHLCheck:
        ld      a,(hl)
        cp      '0'
        jr      nz,showAHLReady
        inc     hl
        djnz    showAHLCheck
showAHLReady:
        bcall(_vputs)
        ret

.end
