;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Passcode routines                                             ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; loadFromPasscode - try to load a cheat or game from a passcode
;   input:  (passcode) = passcode to decode
;   output: none
;   notes:  jumps to wherever it needs based on the result
;------------------------------------------------
loadFromPasscode:
        ; check if a cheat code was entered
        ld      hl,cheatcodes                   ; HL => cheat code data
        ld      b,(hl)                          ; B = number of cheat codes to check
        inc     hl                              ; HL => first cheat code
checkCheatCodes:
        push    bc                              ; save cheat code counter
        ld      bc,10*256+0                     ; B = 10 characters to check; C = clear error flag
        ld      de,passcode                     ; DE => passcode that was entered
checkCheatCodeLoop:
        ld      a,(de)                          ; A = passcode char
        cp      (hl)                            ; compare it to cheat code char
        jr      z,cheatCodeCharOK               ; if they're equal, this char is ok
        inc     c                               ; otherwise, increment the error flag
cheatCodeCharOK:
        inc     de                              ; DE => next passcode char
        inc     hl                              ; HL => next cheatcode char
        djnz    checkCheatCodeLoop
        ld      a,c                             ; A = error flag
        pop     bc                              ; restore cheat code counter
        or      a                               ; check if any errors were registered during this cheat code check
        jp      z,enableCheat                   ; if no errors, this cheat code matched, so enable the cheat
        djnz    checkCheatCodes                 ; otherwise, loop for next cheat code until done
        
        call    decodePasscode
        jp      nc,newPCGame                    ; new game with variables from passcode

invalidPasscode:
#ifdef TI83PLUS
        set     textInverse,(iy+textFlags)      ; write text in white on black
        ld      hl,strInvalidPasscode
        ld      de,43*256+19
        call    vPuts                           ; "Invalid passcode!"
        bcall(_grbufcpy)
        res     textInverse,(iy+textFlags)
#endif
#ifdef TI84PLUSCE
        call    copyVram
        RECT_POS(80,80)
        RECT_SIZE(160,80)
        ld      a,COLOUR_DARKORANGE
        call    drawWindow
        ld      ix,strInvalidPasscode
        ld      de,92
        ld      l,108
        call    printString
        call    vramFlip
#endif
        call    waitKey
        jp      doMenuMain

enableCheat:
        dec     b                               ; B = cheat number
#ifdef TI83PLUS
        ld      a,b
        add     a,a                             ; x2
        add     a,b                             ; x3
        ld      e,a
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      e,b
        ld      d,4
        mlt     de
#endif
        ld      ix,cheatTable
        add     ix,de                           ; IX => cheat flag location information
#ifdef TI83PLUS
        ld      l,(ix)
        ld      h,(ix+1)                        ; HL => flag byte
        ld      a,(ix+2)                        ; A = bit mask
#endif
#ifdef TI84PLUSCE
        ld      hl,(ix)
        ld      a,(ix+3)
#endif
        ld      c,a
        and     %01111111                       ; mask out bit 7 (this is used for the cheat code achievements)
        or      (hl)                            ; apply it to flag byte
        ld      (hl),a                          ; and save it
        bit     7,c
        jr      z,setAchBadCheat
setAchGoodCheat:
        ld      bc,ACH_GOODCHEAT_BIT*256+ACH_GOODCHEAT_ID
        ld      de,ACH_GOODCHEAT_OFFSET
        jr      setAchCheat
setAchBadCheat:
        ld      bc,ACH_BADCHEAT_BIT*256+ACH_BADCHEAT_ID
        ld      de,ACH_BADCHEAT_OFFSET
setAchCheat:
        ld      hl,viewNewAchievement
#ifdef TI83PLUS
        res     textInverse,(iy+textFlags)
#endif
        bcall(ab5_setAchievement)
        jp      doMenuMain

;------------------------------------------------
; decodePasscode - decode a passcode into it's variable values
;   input:  (passcode) = passcode to decode
;   output: CA if passcode invalid
;           game variables set if passcode was valid
;------------------------------------------------
decodePasscode:
#ifdef TI83PLUS
        ld      hl,passcode
#endif
#ifdef TI84PLUSCE
        ; backup passcode
        ld      hl,passcode                     ; HL => start of passcode
        push    hl
        ld      de,pcBuffer
        ld      bc,11
        ldir
        pop     hl
#endif
        ; convert characters back to nibble values
        push    hl                              ; save ptr for later
        ld      b,10                            ; B = 10 characters to convert
        ld      de,passcodeValues               ; DE => passcode char to value conversion table
unmakePCLoop:
        push    hl                              ; save ptr
        ld      a,(hl)                          ; A = char
        sub     'A'                             ; A = entry in passcodeValues to get
        LDHLA()
        add     hl,de                           ; HL => value for nibble
        ld      a,(hl)
        pop     hl                              ; restore ptr
        ld      (hl),a                          ; save value
        inc     hl                              ; update ptr
        djnz    unmakePCLoop
        ; calculate checksum and sub key value from the 8 nibbles at the same time
        dec     hl
        dec     hl                              ; HL => key
        ld      d,(hl)                          ; D = key value
        pop     hl                              ; HL => start of nibble data
        push    hl                              ; save ptr again
        ld      bc,8*256+0                      ; B = 8 nibbles to decode; C = reset checksum
decodePCLoop:
        ld      a,(hl)                          ; A = encoded nibble
        add     a,c
        ld      c,a                             ; update checksum
        ld      a,(hl)                          ; A = encoded nibble
        sub     d                               ; subtract the key value
        and     %00001111                       ; mask out any underflow
        ld      (hl),a                          ; save decoded nibble
        inc     hl                              ; update ptr
        djnz    decodePCLoop
        inc     hl                              ; HL => saved checksum
        ld      a,c                             ; A = calculated checksum
        and     %00001111                       ; mask out any possible overflow
        cp      (hl)                            ; check if they match
        pop     hl                              ; HL => start of nibble data
        jr      nz,decodePasscodeFailed         ; if not, passcode invalid
        ; now put nibbles back together to form 4 sequential bytes, which is our variables (level, weaponTable, credits_LSB, credits_MSB)
        ld      de,passcodeVars                 ; DE => where to store variables as 4 bytes
        push    de                              ; save var ptr
        ld      b,4                             ; B = 4 bytes to create
makePCVarsLoop:
        ld      a,(hl)                          ; A = first nibble of byte
        rlca
        rlca
        rlca
        rlca                                    ; A = first nibble stored in 4 msb's
        inc     hl                              ; HL => second nibble of byte
        or      (hl)                            ; A = full byte
        ld      (de),a                          ; save byte
        inc     hl                              ; update nibble ptr
        inc     de                              ; update byte ptr
        djnz    makePCVarsLoop
        ; save the 4 sequential bytes into their proper variable locations and error-check at the same time
        pop     hl                              ; HL => start of sequential byte data
        ; initially set to 1 player game
        xor     a
        ld      (coop),a
        ld      (p2p),a
        inc     a
        ld      (p1p),a
        ; check to see if it should be 2 player co-op
        bit     7,(hl)
        jr      z,afterPasscodePlayers
        ; set to 2 player game
        ld      (coop),a                        ; (A still equals 1 from before)
        ld      (p2p),a
afterPasscodePlayers:
        ld      a,(hl)
        and     %01111111                       ; A = level (as of v2.x, 0-23)
        ; check to make sure it's not above the highest available level
        cp      NUMLEVELS
        jr      nc,decodePasscodeFailed
        push    hl
        ld      hl,episodeTable+NUMEPISODES-1   ; HL => # levels in last episode
        ld      b,NUMEPISODES-1                 ; B = last episode
levelToEp:
        cp      (hl)                            ; check if passcode level is >= first level offset value for this episode
        jr      nc,calcEpLevel                  ; if so, calculate based on this
        dec     hl                              ; HL => # levels in previous episode
        dec     b                               ; B = previous episode
        jr      levelToEp
calcEpLevel:
        sub     (hl)                            ; A = actual level number
        ld      (level),a
        ld      a,b                             ; A = episode
        ld      (episode),a
        pop     hl                              ; restore passcode parsing ptr
        inc     hl
        ld      a,(hl)                          ; A = weapon table
        ld      (weaponTable),a                 ; save it
        and     %11000000                       ; check that no junk data is stored
        jr      nz,decodePasscodeFailed         ; if so, passcode invalid
        inc     hl
        ld      e,(hl)                          ; E = credits LSB
        inc     hl
        ld      d,(hl)                          ; D = credits MSB
#ifdef TI84PLUSCE
        call    _setDEUto0
#endif
        ld      (credits),de                    ; save it
        or      a
        ret
decodePasscodeFailed:
        scf
        ret

;------------------------------------------------
; makePasscode - generate a passcode
;   input:  none
;   output: HL => passcode string (NULL terminated)
;------------------------------------------------
makePasscode:
        ; first, save the 3 variables (4 bytes) consecutively
        ld      hl,passcodeVars                 ; HL => where to store variables consecutively
        push    hl                              ; save ptr
        push    hl                              ; and again to preserve it through the level calculation
#ifdef TI83PLUS
        ld      hl,(episode)
        ld      h,0
#endif
#ifdef TI84PLUSCE
        ld      a,(episode)
        LDHLA()
#endif
        ld      de,episodeTable
        add     hl,de                           ; HL => value to add to calculator passcode level value
        ld      a,(level)
        add     a,(hl)                          ; A = passcode level value
        ld      b,a                             ; save it in B
        ld      a,(p1p)
        ld      hl,p2p
        add     a,(hl)
        cp      2                               ; 2 active players?
        ld      a,b                             ; A = passcode level id
        jr      nz,savePCLevel                  ; if not, nothing to do
        or      %10000000                       ; otherwise set bit 7 to indicate a 2-player passcode
savePCLevel:
        pop     hl                              ; restore ptr
        ld      (hl),a                          ; save level
        inc     hl
        ld      a,(weaponTable)
        ld      (hl),a                          ; save weapon table
        inc     hl
        ld      de,(credits)
        ld      (hl),e                          ; save credits LSB
        inc     hl
        ld      (hl),d                          ; save credits MSB
; now, split the 4 bytes into 8 bytes, effectively stored as 8 nibbles (each byte only uses the 4 lsb's)
        pop     de                              ; DE => start of consecutive vars
        ld      hl,passcode                     ; HL => where to store them
        push    hl                              ; save passcode data ptr
        ld      b,4                             ; B = 4 bytes to split
splitPCVarsLoop:
        ld      a,(de)                          ; A = first byte
        rrca
        rrca
        rrca
        rrca
        and     %00001111                       ; A = first nibble of byte
        ld      (hl),a                          ; save it
        inc     hl
        ld      a,(de)
        and     %00001111                       ; A = second nibble of byte
        ld      (hl),a                          ; save it
        inc     hl
        inc     de
        djnz    splitPCVarsLoop
; get a random key 0-15
        ld      b,16
        bcall(ab5_getRandom)                    ; A = random between 0-15 inclusive
        ld      (hl),a                          ; save key
; now add key to all 8 nibbles & calculate checksum as well
        ld      d,a                             ; D = key
        pop     hl                              ; HL => start of nibble data
        push    hl                              ; save ptr again
        ld      bc,8*256+0                      ; B = 8 nibbles to encode; C = reset checksum counter
encodePCLoop:
        ld      a,(hl)                          ; A = nibble
        add     a,d                             ; apply random key
        and     %00001111                       ; mask to get rid of any overflow
        ld      (hl),a                          ; save encoded nibble
        add     a,c
        ld      c,a                             ; update checksum
        inc     hl                              ; update ptr
        djnz    encodePCLoop
        inc     hl                              ; HL => where to store checksum
        ld      a,c
        and     %00001111                       ; mask checksum to get rid of any overflow
        ld      (hl),a                          ; save checksum
; finally, convert encoded nibbles into characters, which is the passcode
        pop     hl                              ; HL => start of passcode data
        push    hl                              ; save ptr again
        ld      b,10                            ; B = 10 nibbles to convert
        ld      de,passcodeChars                ; DE => passcode value to char conversion table
makePCLoop:
        push    hl                              ; save ptr
#ifdef TI83PLUS
        ld      l,(hl)
        ld      h,0                             ; HL = nibble data
#endif
#ifdef TI84PLUSCE
        ld      a,(hl)
        LDHLA()
#endif
        add     hl,de                           ; HL => char to use
        ld      a,(hl)
        pop     hl                              ; restore ptr
        ld      (hl),a                          ; save char
        inc     hl                              ; update ptr
        djnz    makePCLoop
        ld      (hl),0                          ; NULL terminator
        pop     hl                              ; HL => generated passcode string
        ret

#ifdef TI84PLUSCE
;------------------------------------------------
; loadPasscodeFile - load the passcode appvar into the buffer, or if it's not found, reset the buffer
;   input:  none
;   output: none
;------------------------------------------------
loadPasscodeFile:
        ld      hl,appvarCodes
        MOVE9()
        call    _chkfindsym
        jr      c,lpfNotFound
        call    _chkInRam
        jr      z,lpfInRam
        call    _arc_unarc
        jr      loadPasscodeFile
lpfInRam:
        inc     de
        inc     de
        ex      de,hl                           ; HL => start of data
        ld      de,codesBuffer
        ld      c,(hl)
        ld      b,10
        mlt     bc
        inc     bc
        ldir
        ret
lpfNotFound:
        xor     a
        ld      (codesBuffer),a
        ret

;------------------------------------------------
; savePasscodeFile - create an appvar to save passcodes stored in the buffer
;   input:  none
;   output: none
;------------------------------------------------
savePasscodeFile:
        ld      hl,appvarCodes
        MOVE9()                                 ; (OP1) = name of appvar to be checked for and created if necessary
        call    _chkfindsym
        jr      c,spfCreate
        call    _delvararc
spfCreate:
        ld      a,(codesBuffer)
        or      a
        ret     z                               ; if no passcodes saved, no need to create the appvar
        ld      l,a
        ld      h,10
        mlt     hl
        inc     hl                              ; HL = space required for appvar
        push    hl
        call    _createAppVar
        inc     de
        inc     de                              ; DE => where to store passcodes in appvar (after size bytes)
        ld      hl,codesBuffer
        pop     bc                              ; BC = # bytes to copy
        ldir
        ld      hl,appvarCodes
        MOVE9()
        call    _arc_unarc
        ret

;------------------------------------------------
; savePasscode - try to save last passcode given
;   input:  none
;   output: none
;------------------------------------------------
savePasscode:
        ld      hl,codesBuffer
        ld      a,(hl)
        cp      MAX_CODES
        jr      nc,spError
        inc     (hl)
        inc     hl
        ld      e,a
        ld      d,10
        mlt     de
        add     hl,de
        ex      de,hl
        ld      hl,passcode
        ld      bc,10
        ldir
        ld      de,strPasscodeSaved
        jr      spFinish
spError:
        ld      de,strPasscodeFailed
spFinish:
        ld      hl,strPasscodeManager
        xor     a
        jp      showInfoBox

;------------------------------------------------
; viewPasscodes - view stored passcodes
;   input:  none
;   output: none
;------------------------------------------------
viewPasscodes:
        ld      a,(codesBuffer)
        or      a
        jp      z,menuLoop                      ; nothing to view if there are no stored passcodes
        xor     a
        ld      (pcPage),a
        ld      (pcNum),a
        
vpLoop:
        call    vbufClear
        
        ; show page of passcodes
        ld      hl,(pcPage)
        ld      h,CODES_PER_PAGE
        mlt     hl                              ; L = number of first passcode to be shown
        ld      a,(codesBuffer)
        sub     l
        cp      CODES_PER_PAGE
        jr      c,vpSetDispCount
        ld      a,CODES_PER_PAGE
vpSetDispCount:
        ld      b,a                             ; B = number of passcodes to show this page
        ld      (pcPageCount),a
        ld      c,0
        
        ld      h,10
        mlt     hl
        ld      de,codesBuffer+1
        add     hl,de                           ; HL => first passcode to show on this page
        
vpDispLoop:
        ld      a,c
        push    bc
        ld      de,passcode
        ld      bc,10
        ldir
        ex      de,hl
        ld      (hl),0
        push    de
        ld      hl,pcNum
        cp      (hl)
        ld      l,a
        ld      a,COLOUR_WHITE
        jr      nz,vpColourSet
        ld      a,$FF
vpColourSet:
        push    af
        push    hl
        call    decodePasscode
        pop     hl
        pop     af
        ld      h,28
        mlt     hl
        push    hl
        ld      de,2
        ld      ix,pcBuffer
        call    printStringC                    ; print passcode
        pop     bc
        push    bc
        ld      de,232
        ld      hl,(credits)
        call    showHL
        ld      ix,strCr
        call    printString                     ; print credits
        call    getEpLvlOffset
        push    hl
        ld      hl,(pDatFile)
        ld      de,pcLevelTable
        add     hl,de
        pop     de
        add     hl,de
        ld      de,(hl)
        ld      ix,(pDatFile)
        add     ix,de
        pop     hl
        ld      a,12
        add     a,l
        ld      l,a
        ld      de,16
        call    printString
        
        pop     hl
        pop     bc
        inc     c
        djnz    vpDispLoop
        
        ld      a,COLOUR_WHITE
        ld      (__psColour),a
        ld      ix,strPasscodeKeys
        ld      de,12
        ld      l,228
        call    printString
        
        call    vramFlip
        ld      hl,frame
        ld      a,(hl)
        inc     a
        and     $0F
        ld      (hl),a
        call    updateCursorFlash
        call    _getcsc
        cp      GK_MODE
        jp      z,mainMenu
        ld      hl,pcPageCount
        ld      de,pcNum
        cp      GK_UP
        jr      z,vpUp
        cp      GK_DOWN
        jr      z,vpDown
        ld      hl,codesBuffer
        ld      de,pcPage
        cp      GK_LEFT
        jr      z,vpPrevPage
        cp      GK_RIGHT
        jr      z,vpNextPage
        cp      GK_2ND
        jr      z,vpSelect
        cp      GK_ENTER
        jr      z,vpSelect
        cp      GK_DEL
        jr      z,vpDelete
        jp      vpLoop
        
vpUp:
        ld      a,(de)
        dec     a
        jp      m,vpLastCode
vpSaveNum:
        ld      (de),a
        jp      vpLoop
vpLastCode:
        ld      a,(hl)
        dec     a
        jr      vpSaveNum
vpDown:
        ld      a,(de)
        inc     a
        cp      (hl)
        jr      c,vpSaveNum
        xor     a
        jr      vpSaveNum
        
vpPrevPage:
        ld      a,(de)
        dec     a
        jp      m,vpLastPage
vpSavePage:
        ld      (de),a
        xor     a
        ld      (pcNum),a
        jp      vpLoop
vpLastPage:
        ld      a,(hl)
        dec     a
        srl a \ srl a \ srl a
        jr      vpSavePage
vpNextPage:
        ld      a,(hl)
        dec     a
        srl a \ srl a \ srl a
        inc     a
        ld      c,a
        ld      a,(de)
        inc     a
        cp      c
        jr      c,vpSavePage
        xor     a
        jp      vpSavePage
        
vpSelect:
        ld      hl,(pcPage)
        ld      h,CODES_PER_PAGE
        mlt     hl
        ld      a,(pcNum)
        add     a,l
        ld      e,a
        ld      d,10
        mlt     de
        ld      hl,codesBuffer+1
        add     hl,de
        ld      de,passcode
        ld      bc,10
        ldir
        jp      loadFromPasscode
        
vpDelete:
        ld      hl,strDeletePasscode
        ld      de,strAreYouSure
        ld      a,1
        call    showInfoBox
        jp      nc,vpLoop
        
        ; defrag codesBuffer
        ld      hl,(pcPage)
        ld      h,CODES_PER_PAGE
        mlt     hl
        ld      a,(pcNum)
        add     a,l
        ld      c,a
        ld      l,a
        ld      h,10
        mlt     hl
        ld      de,codesBuffer+1
        add     hl,de
        push    hl
        ld      de,10
        add     hl,de                           ; HL => where to start copying from
        pop     de                              ; DE => where to copy to
        ld      a,(codesBuffer)
        sub     c
        dec     a
        jr      z,vpDeleteFinish
        ld      c,a
        ld      b,10
        mlt     bc
        ldir
vpDeleteFinish:
        ; decrement # of stored codes
        ld      hl,codesBuffer
        dec     (hl)
        jp      z,mainMenu                      ; if none left, go back to main menu
        jp      viewPasscodes
#endif

.end
