;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Pause routines                                                ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; teacherKey - turn the calc off, and when turned back on, go to the pause screen
;   input:  none
;   output: none
;------------------------------------------------
teacherKey:
        bcall(_clrscrnfull)                     ; clear LCD
        ld      a,1
        out     (3),a                           ; enable [ON] key & low power mode
        halt                                    ; low power mode (turn off LCD, etc.) until [ON] is pressed
        ld      a,11
        out     (3),a                           ; turn hardware timer back on and disable low power mode
        ; fall through to pause routine

;------------------------------------------------
; pauseScreen - display the pause screen and wait for either [ENTER] or [CLEAR]
;   input:  none
;   output: none
;------------------------------------------------
pauseScreen:
        call    resetZOffset                    ; ensure LCD Z-offset is reset
        res     apdable,(iy+apdflags)           ; disable apd
        bcall(ab5_saveVideo)                    ; save gbuf
pauseWait:
        ld      a,KG_TOP
        bcall(ab5_directIn)
        bit     DI_MODE,a                       ; [MODE]?
        jr      z,pauseWait                     ; don't go into pause screen until [MODE] is released
        bcall(GET_KEY)
        bcall(GET_KEY)                          ; clear key buffer
        xor     a
        ld      (pauseFrame),a                  ; reset pause screen frame counter
pauseLoop:
        bcall(_grbufclr)
        ld      hl,pauseFrame
        inc     (hl)                            ; increment counter
        ; write text
        ld      hl,strPaused                    ; HL => start of string data
        call    showStrings                     ; show strings
        ld      a,(pauseFrame)
        bit     5,a
        ld      hl,strPausedBottom1
        jr      z,pauseBottomText
        ld      hl,strPausedBottom2
pauseBottomText:
        call    showStrings
        ; draw pause screen sprites
        ld      hl,sprPauseTable
        ld      b,4
pauseSprLoop:
        push    bc                              ; save loop counter
        ld      a,(hl)                          ; A = x pos
        inc     hl                              ; HL => sprite ptr
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE => sprite
        inc     hl                              ; HL => next sprite data
        push    hl                              ; save data ptr
        ex      de,hl                           ; HL => sprite on flash page
        call    spriteToRam                     ; DE => sprite in ram
        ld      l,8                             ; L = y pos
        bcall(ab5_putSprite)                    ; draw sprite
        pop     hl                              ; restore data ptr
        pop     bc                              ; restore loop counter
        djnz    pauseSprLoop
        
        call    pausePlayerData                 ; load stack with pointers to player data to show
        ; show [P1] or [P2] at top left
        pop     hl                              ; HL => string
        call    showStrings
        ; show health meter
        pop     hl
        ld      a,(hl)                          ; A = health
        ld      hl,P1HMETER
        bcall(ab5_displayMeter)
        ; show ammo meter
        pop     hl
        ld      a,(hl)
        srl     a                               ; A = ammo/2
        ld      hl,AMETER
        bcall(ab5_displayMeter)
        ; show lives
        pop     hl
        ld      a,(hl)                          ; A = lives
        cp      10
        jr      c,pauseLives                    ; if under 10, all ok
        inc     a                               ; check if lives=255 (meaning that player is dying with no lives left)
        ld      a,9                             ; if not, otherwise show 9
        jr      nz,pauseLives
        xor     a                               ; if player is dying with no lives left, show 0 lives
pauseLives:
        ld      l,a
        ld      h,0
        ld      de,50*256+243
        call    setShowHL
        ; show ammo clips
        pop     hl
        ld      l,(hl)
        ld      h,0                             ; HL = ammo clips
        ld      de,50*256+251
        call    setShowHL
        ; show keys
        pop     hl
        ld      l,(hl)
        ld      h,0                             ; HL = keys
        ld      de,16*256+7
        call    setShowHL
        ; show score
        pop     hl
        LDHLIND()                               ; HL = score
        ld      de,8*256+75
        call    setShowHL
        ; show credits
        ld      hl,(credits)
        ld      de,8*256+35
        call    setShowHL

        ; show status of weapons & radar
        ld      b,6                             ; B = number of bits to check
        ld      a,(weaponTable)
        ld      c,a                             ; C = bits to check
        ld      hl,strYesNoTable                ; HL => penrow data
pauseWeaponLoop:
        srl     c                               ; CA = weapon owned; NC = weapon not owned
        push    bc                              ; save counter & bit data
        ld      d,(hl)                          ; D = penrow
        push    hl                              ; save penrow data ptr
        ld      hl,strYes
        jr      c,pauseWriteWeaponStatus        ; if weapon owned, display "YES"
        ld      hl,strNo
pauseWriteWeaponStatus:
        ld      e,(hl)                          ; E = pencol
        inc     hl                              ; HL => string
        call    vPuts                           ; display either "YES" or "NO"
        pop     hl                              ; restore penrow data ptr
        inc     hl                              ; next entry
        pop     bc                              ; restore counter & bit data
        djnz    pauseWeaponLoop

        ; invert top and bottom text lines
        ld      hl,gbuf                         ; HL => where to start inverting
        ld      b,7                             ; B = num gbuf lines to invert
        call    invertLine                      ; invert B lines starting from HL
        ld      hl,gbuf+(57*12)
        ld      b,7
        call    invertLine
        bcall(_grbufcpy)
        bcall(GET_KEY)
        push    af
        cp      GK_PLUS                         ; [+]?
        call    z,contrastUp
        pop     af
        push    af
        cp      GK_MINUS                        ; [-]?
        call    z,contrastDown
        pop     af
        cp      GK_STAT                         ; [STAT]?
        jr      z,pauseViewAchievements         ; if so, view achievements
        cp      GK_2ND                          ; [2nd]?
        jr      z,pauseFinished
        cp      GK_ENTER                        ; [ENTER]?
        jr      z,pauseFinished
        cp      GK_MODE                         ; [MODE]?
        jr      z,pauseFinished
        cp      GK_CLEAR                        ; [CLEAR]?
        jp      nz,pauseLoop
pauseConfirm:
        ld      hl,gbuf
        ld      b,7
        call    fillLine                        ; block out top text bar in pause screen
        ld      hl,gbuf+(57*12)
        ld      b,7
        call    fillLine                        ; block out bottom text bar
        ld      hl,strRetreatTitle
        ld      de,strAreYouSure
        ld      a,1                             ; prompt box
        call    showInfoBox                     ; prompt to confirm if quitting
        jp      nc,pauseLoop                    ; if "No" selected return to pause loop
        pop hl \ pop hl \ pop hl                ; pop bcall stack data
        pop     hl                              ; pop call stack data
        bcall(ab5_restoreVideo)                 ; restore gbuf from start of routine
        bcall(ab5_drawPlayers)                  ; draw the players to gbuf
        bcall(_grbufcpy)                        ; show game screen before exploding
        jp      levelExplode
pauseFinished:
        bcall(ab5_restoreVideo)                 ; restore gbuf from start of routine
        ld      b,INTO_GAME_DELAY
        jp      delay                           ; short delay before returning to gameplay
pauseViewAchievements:
        bcall(ab5_viewAchievements)
        jp      pauseLoop

;------------------------------------------------
; pausePlayerData - load pointers to player data to show this frame to the stack
;   input:  none
;   output: none
;------------------------------------------------
pausePlayerData:
        pop     bc                              ; BC = ret address (needs to be put back on the stack later)
        ld      a,(p1p)
        or      a                               ; player 1 active?
        jr      z,ppdP2                         ; if not, always show player 2
        ld      a,(p2p)
        or      a
        jr      z,ppdP1                         ; if player 2 not active, always show player 1
        ; otherwise both players are active, so check pauseFrame
        ld      a,(pauseFrame)
        bit     5,a
        jr      nz,ppdP2
ppdP1:
        ; push pointers to player 1 data to the stack
        ld      hl,p1s                          ; score
        push    hl
        ld      hl,p1k                          ; keys
        push    hl
        ld      hl,p1a+1                        ; ammo clips
        push    hl
        ld      hl,p1l                          ; lives
        push    hl
        ld      hl,p1a                          ; ammo in current clip
        push    hl
        ld      hl,p1h                          ; health
        push    hl
        ld      hl,strP1                        ; string for top left
        push    hl
        jr      ppdDone
ppdP2:
        ld      hl,p2s
        push    hl
        ld      hl,p2k
        push    hl
        ld      hl,p2a+1
        push    hl
        ld      hl,p2l
        push    hl
        ld      hl,p2a
        push    hl
        ld      hl,p2h
        push    hl
        ld      hl,strP2
        push    hl
ppdDone:
        push    bc                              ; put the ret address back on the stack
        ret

.end
