;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Shooting routines                                             ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; playerShoot - try to create a new player bullet
;   input:  A = "or" value to use for miscFlags to denote weapon blast
;   output: none
;------------------------------------------------
playerShoot:
        ld      (tempB),a                       ; save "or" value
        ld      hl,frame
        bit     0,(hl)
        ret     nz                              ; player can only shoot every 2nd frame
        ld      hl,e3CheatFlags
        bit     ammoCheat,(hl)
        jr      z,checkAmmo                     ; if cheat not enabled, need to check ammo
        ld      ix,bulletTable
        call    findEmptyBulletEntry
        ret     c
        ld      hl,(pPlayer)
        jr      checkWeapon
checkAmmo:
        ld      hl,(pPlayer)
        ld      de,PA
        add     hl,de
        LDHLIND()                               ; HL = player ammo
        ld      a,l
        or      h
        ret     z                               ; can't shoot if no ammo
        ; check bulletTable for an empty entry
        ld      ix,bulletTable                  ; IX => start of table
        call    findEmptyBulletEntry            ; try to find an empty entry
        ret     c                               ; if none found, leave
        dec     hl
        dec     hl                              ; decrement ammo (HL still holds the value from earlier)
        bit     7,h                             ; check if ammo went below 0?
        jr      z,saveAmmo                      ; if not, go straight to saving it
        ld      hl,0                            ; otherwise set it to 0
saveAmmo:
        ex      de,hl
        ld      hl,(pPlayer)
        push    hl
        ld      bc,PA
        add     hl,bc
        ld      (hl),e
        inc     hl
        ld      (hl),d                          ; new ammo amount saved
        pop     hl
checkWeapon:
        push    hl
        ld      de,PW
        add     hl,de
        ld      a,(hl)                          ; A = weapon type
        ld      de,PD-PW
        add     hl,de
        ld      c,(hl)                          ; C = player direction
        pop     hl                              ; HL => current player x coord
        push    af
        call    newBullet                       ; set up new bullet
        pop     af
        dec     a                               ; machine gun bullet?
        jr      nz,psClearAchFlag               ; if not, skip machine gun blast
        ; set blast effect flag for drawBullets
        ld      hl,miscFlags
        ld      a,(tempB)                       ; A = "or" value
        or      (hl)                            ; set appropriate bit
        ld      (hl),a
        ret
psClearAchFlag:
        ; clear achievement flag
        xor     a
        ld      (aflagHookCrook),a              ; clear the "BY HOOK OR BY CROOK" flag (used during boss fights)
        ret

;------------------------------------------------
; findEmptyBulletEntry - find an empty entry in a half section of bulletTable (only half can be used per player)
;   input:  IX => start of section of bulletTable
;   output: IX => empty entry (if one found)
;           NC if entry found, CA if not found
;------------------------------------------------
findEmptyBulletEntry:
        ld      b,NUM_BULLETS/2                 ; B = num entries to check (each player only has access to half the bulletTable)
        ld      de,BULLET_SIZE                  ; DE = size of each entry
bulletEntrySearch:
        ld      a,(ix)                          ; check type
        or      a                               ; is it empty?
        ret     z                               ; if so, leave (carry will be clear from the "or a")
        add     ix,de                           ; IX => next entry
        djnz    bulletEntrySearch
        scf                                     ; set carry (no entries found)
        ret

;------------------------------------------------
; newBullet - set up a new bullet
;   input:  IX => entry in bulletTable to use
;           A = weapon type
;           C = player direction
;           HL => player x coord
;   output: none
;------------------------------------------------
newBullet:
        push    hl                              ; save x coord ptr
        ld      (ix),a                          ; store it
        dec     a                               ; weapon type will now be between 0-4
#ifdef TI83PLUS
        add     a,a
        add     a,a
        add     a,a
        add     a,a
        ld      d,a                             ; D = offset to type in bulletOffsetTable
        ld      a,c                             ; A = player direction
        ld      (ix+1),a                        ; save it as bullet direction
        add     a,a
        add     a,a                             ; A = direction offset for bulletOffsetTable
        add     a,d                             ; A = offset to coords in bulletOffsetTable
        ld      h,0
        ld      l,a
#endif
#ifdef TI84PLUSCE
        ld      l,a
        ld      h,24
        mlt     hl
        ld      a,c
        ld      (ix+1),a
        ld      e,a
        ld      d,6
        mlt     de
        add     hl,de
#endif
        ld      de,bulletOffsetTable
        add     hl,de                           ; HL => coords to use in bulletOffsetTable
#ifdef TI83PLUS
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = x coord offset
        inc     hl
#endif
#ifdef TI84PLUSCE
        call    _lddehl
#endif
        ex      (sp),hl                         ; HL => player x coord, stack contains bulletOffsetTable ptr
#ifdef TI83PLUS
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = player x coord
        inc     hl                              ; HL => player y coord
#endif
#ifdef TI84PLUSCE
        push    de
        call    _lddehl
        push de \ pop bc
        pop     de
#endif
        ex      (sp),hl                         ; HL => bulletOffsetTable ptr, stack contains player y coord ptr
        ex      de,hl                           ; HL = x coord offset, DE => bulletOffsetTable ptr
        add     hl,bc                           ; HL = bullet x coord
        ld      (ix+2),l                        ; save x coord LSB
        ld      (ix+3),h                        ; save x coord MSB
        ex      de,hl                           ; HL => bulletOffsetTable ptr
#ifdef TI83PLUS
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = y coord offset
#endif
#ifdef TI84PLUSCE
        call    _lddehl
#endif
        pop     hl                              ; HL => player y coord
#ifdef TI84PLUSCE
        ld      bc,0
#endif
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = player y coord
        ex      de,hl                           ; HL = y coord offset
        add     hl,bc                           ; HL = bullet y coord
        ld      (ix+4),l                        ; save y coord LSB
        ld      (ix+5),h                        ; save y coord MSB
        ld      (ix+6),0                        ; reset animation counter
        ld      a,(bulletSlide)
        ld      (ix+7),a                        ; set slide value (used for flame arc and plasma rifle)
        inc     a
        cp      3                               ; max value is 2
        jr      nz,nbSaveSlide
        xor     a
nbSaveSlide:
        ld      (bulletSlide),a
        ret

;------------------------------------------------
; drawBullets - draw player bullets
;   input:  none
;   output: none
;------------------------------------------------
drawBullets:
        ld      hl,miscFlags
        ld      ix,p1x
        bit     p1Blast,(hl)
        push    hl
        call    nz,drawGunBlast                 ; if player 1 shot this frame, show the effect
        pop     hl
        ld      ix,p2x
        bit     p2Blast,(hl)
        call    nz,drawGunBlast                 ; same for player 2
        ; now draw the bullets
        ld      ix,bulletTable                  ; IX => start of data
        ld      b,NUM_BULLETS                   ; B = number of bullets to draw
drawBulletsLoop:
        push    bc                              ; save counter
        ld      a,(ix+0)                        ; A = type
        or      a                               ; if 0, it's an empty entry
        jr      z,endDrawBulletsLoop
        dec     a                               ; A = type {0-4}
        jr      z,endDrawBulletsLoop            ; if type = 0, no sprite to draw
#ifdef TI83PLUS
        add     a,a
        ld      e,a
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      e,a
        ld      d,3
        mlt     de
#endif
        ld      hl,bulletSpriteTable
        add     hl,de                           ; HL => address of sprites for this bullet type
        LDHLIND()                               ; HL => sprites for this bullet type
        call    getBulletScreenCoords           ; B = x coord, C = y coord (on screen)
        ld      a,(ix+0)
        cp      3                               ; is it a flame arc bullet?
        jr      z,drawBulletAnim                ; if so, calculate animation
        cp      5                               ; ditto if flame thrower bullet
        jr      z,drawBulletAnim
#ifdef TI83PLUS
        ld      a,(ix+1)                        ; A = bullet direction
        add     a,a
        add     a,a
        add     a,a
        ld      e,a
        ld      d,0                             ; DE = direction offset to sprite
#endif
#ifdef TI84PLUSCE
        ld      e,(ix+1)
        ld      d,128
        mlt     de
        ex      de,hl
        add     hl,hl
#endif
        add     hl,de                           ; HL => sprites for this bullet & direction
drawBulletDraw:
        push    ix                              ; save bulletTable ptr
#ifdef TI83PLUS
        call    putClippedSprite                ; display bullet
#endif
#ifdef TI84PLUSCE
        ld      e,b
        ld      d,c
        call    drawClippedTransSprite
#endif
        pop     ix                              ; restore bulletTable ptr
endDrawBulletsLoop:
        pop     bc                              ; restore counter
        ld      de,BULLET_SIZE
        add     ix,de                           ; IX => next entry
        djnz    drawBulletsLoop
        ret

drawBulletAnim:
#ifdef TI83PLUS
        ld      a,(ix+6)
        add     a,a
        add     a,a
        add     a,a
        ld      e,a
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ex      de,hl
        ld      l,(ix+6)
        ld      h,128
        mlt     hl
        add     hl,hl
#endif
        add     hl,de                           ; HL => sprite
        jr      drawBulletDraw

;------------------------------------------------
; drawGunBlast - draw the blast effect when a player shoots
;   input:  IX => player data
;   output: none
;------------------------------------------------
drawGunBlast:
#ifdef TI83PLUS
        ld      a,(ix+PD)
        add     a,a
        add     a,a
        ld      e,a
        ld      d,0                             ; DE = offset to data in bulletOffsetTable
        add     a,a
        push    af                              ; save pd * 8 for later
#endif
#ifdef TI84PLUSCE
        ld      e,(ix+PD)
        ld      d,6
        mlt     de
#endif
        ld      hl,bulletOffsetTable
        add     hl,de
        ex      de,hl                           ; DE => required data in bulletOffsetTable
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+PX)
        ld      h,(ix+PX+1)
        ld      bc,(screenX)
        or      a
        sbc     hl,bc                           ; HL = player x on screen
        ex      de,hl                           ; DE = player x on screen; HL => required data in bulletOffsetTable
        push    hl                              ; save ptr to bulletOffsetTable
        LDHLIND()                               ; HL = x offset
        add     hl,de                           ; L = x coord to display blast effect at
        ld      b,l                             ; B = x coord to display at
        pop     de                              ; restore ptr to bulletOffsetTable
        push    bc                              ; save x coord to display at
#ifdef TI84PLUSCE
        inc     de
#endif
        inc     de
        inc     de                              ; DE => y data
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+PY)
        ld      h,(ix+PY+1)
        ld      bc,(screenY)
        or      a
        sbc     hl,bc                           ; HL = player y on screen
        ex      de,hl                           ; DE = player y on screen; HL => required data in bulletOffsetTable
        LDHLIND()                               ; HL = y offset
        add     hl,de
        pop     bc                              ; B = x coord to display at
        ld      c,l                             ; C = y coord to display at
#ifdef TI83PLUS
        pop     af                              ; A = pd * 8 (from earlier)
        ld      e,a
        ld      d,0                             ; DE => offset to sprite data
        ld      hl,sprMachineGun
        add     hl,de                           ; HL => sprite to display
        jp      putClippedSprite                ; draw blast effect and ret
#endif
#ifdef TI84PLUSCE
        ld      l,(ix+PD)
        ld      h,128
        mlt     hl
        add     hl,hl
        ld      de,sprMachineGun
        add     hl,de
        ld      e,b
        ld      d,c
        jp      drawTransSprite
#endif

;------------------------------------------------
; moveBullets - move player bullets
;   input:  none
;   output: none
;------------------------------------------------
moveBullets:
        ld      ix,collideObject2
        call    loadScreenCollideObjectIX       ; will be used to check if bullets are still on screen
        ld      hl,bulletCentreTable
        ld      (pCentreTable),hl               ; set pointer value
        xor     a
        ld      (fBulletCat),a                  ; set bullet category to "player"
        ld      ix,bulletTable                  ; IX => start of data
        ld      b,NUM_BULLETS                   ; B = number of bullets to move
moveBulletsLoop:
        push    bc                              ; save counter
        ld      a,(ix+0)                        ; A = type
        or      a                               ; if 0, it's an empty entry
        jr      z,endMoveBulletsLoop
        cp      3                               ; is it a flame arc bullet?
        jr      z,moveBulletAnim                ; if so, update enimation every frame
        cp      4                               ; plasma rifle?
        jr      z,moveBulletAnim                ; ditto (plasma rifle isn't actually animated, but the slide mechanic relies on the animation counter being active)
        ld      a,(frame)
        and     %00000011                       ; only update animation every 4th frame
        jr      nz,moveBulletMove
moveBulletAnim:
        ld      a,(ix+6)                        ; A = animation counter
        inc     a                               ; increment it
        and     %00000111                       ; can only be 0-7
        ld      (ix+6),a                        ; save it
moveBulletMove:
        di
        push    iy
        push    ix
        pop     iy                              ; IY => bulletTable entry
        call    loadBulletCollideObject1
        pop     iy
        ei
        call    checkCollision                  ; check if bullet is "colliding" with screen
        jr      c,bulletOnScreen                ; if so, continue
        ld      (ix),0                          ; terminate bullet
        jr      endMoveBulletsLoop
bulletOnScreen:
        ld      l,(ix+1)
#ifdef TI83PLUS
        ld      h,0                             ; HL = direction
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,3
        mlt     hl
#endif
        ld      de,bulletMoveTable
        add     hl,de
        LDHLIND()                               ; HL => routine to move bullet
        ld      de,endMoveBulletsLoop           ; DE => where to ret to after moving bullet
        push    de                              ; push ret address to stack
        jp      (hl)                            ; jump to moving routine
endMoveBulletsLoop:
        pop     bc                              ; restore counter
        ld      de,BULLET_SIZE
        add     ix,de                           ; IX => next entry
        djnz    moveBulletsLoop
        ret

;------------------------------------------------
; bullet moving routines (used for both player & enemy bullets)
;------------------------------------------------
moveBulletUp:
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+4)
        ld      d,(ix+5)                        ; DE = bullet y
        dec     de
        dec     de                              ; decrement * 2
        bit     7,d                             ; check if it went below 0
        jp      nz,removeBullet                 ; if so, it's off the level, so remove the bullet
        ld      (ix+4),e
        ld      (ix+5),d                        ; save new bullet y
        ld      a,(ix+0)
        cp      3
        jr      z,mbuSlide
        cp      4
        jp      nz,moveBulletCheck
mbuSlide:
        ld      a,(ix+6)
        dec     a
        and     %00000011
        jp      nz,moveBulletCheck
        ld      e,(ix+2)
        ld      d,(ix+3)
        ld      a,(ix+7)
        dec     a
        jp      z,moveBulletCheck
        dec     a
        jr      z,bulletSlideLeft
        jr      bulletSlideRight
moveBulletDown:
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+4)
        ld      d,(ix+5)                        ; DE = bullet y
        inc     de
        inc     de
        ld      hl,(levelHeight)
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; HL = level height in pixels
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        CPHLDE()                                ; check if bullet has gone off bottom edge of level
        jp      c,removeBullet                  ; if so, remove the bullet
        ld      (ix+4),e
        ld      (ix+5),d                        ; save new bullet y
        ld      a,(ix+0)
        cp      3
        jr      z,mbdSlide
        cp      4
        jp      nz,moveBulletCheck
mbdSlide:
        ld      a,(ix+6)
        dec     a
        and     %00000011
        jp      nz,moveBulletCheck
        ld      e,(ix+2)
        ld      d,(ix+3)
        ld      a,(ix+7)
        dec     a
        jp      z,moveBulletCheck
        dec     a
        jr      z,bulletSlideRight
bulletSlideLeft:
        dec     de
        jr      bulletSlideSaveX
bulletSlideRight:
        inc     de
bulletSlideSaveX:
        ld      (ix+2),e
        ld      (ix+3),d
        jp      moveBulletCheck
moveBulletLeft:
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+2)
        ld      d,(ix+3)                        ; DE = bullet x
        dec     de
        dec     de
        bit     7,d                             ; check if it went under 0
        jr      nz,removeBullet                 ; if so, it's off the level, so remove the bullet
        ld      (ix+2),e
        ld      (ix+3),d                        ; save new bullet x
        ld      a,(ix+0)
        cp      3
        jr      z,mblSlide
        cp      4
        jr      nz,moveBulletCheck
mblSlide:
        ld      a,(ix+6)
        dec     a
        and     %00000011
        jr      nz,moveBulletCheck
        ld      e,(ix+4)
        ld      d,(ix+5)
        ld      a,(ix+7)
        dec     a
        jr      z,moveBulletCheck
        dec     a
        jr      z,bulletSlideDown
        jr      bulletSlideUp
moveBulletRight:
#ifdef TI84PLUSCE
        ld      de,0
#endif
        ld      e,(ix+2)
        ld      d,(ix+3)                        ; DE = bullet x
        inc     de
        inc     de
        ld      hl,(levelWidth)
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; HL = level width in pixels
#endif
#ifdef TI84PLUSCE
        ld      h,8
        mlt     hl
#endif
        CPHLDE()                                ; check if bullet has gone off right edge of level
        jr      c,removeBullet                  ; if so, remove the bullet
        ld      (ix+2),e
        ld      (ix+3),d                        ; save new bullet x
        ld      a,(ix+0)
        cp      3                               ; flame arc?
        jr      z,mbrSlide
        cp      4                               ; plasma rifle?
        jr      nz,moveBulletCheck
mbrSlide:
        ld      a,(ix+6)
        dec     a                               ; this ensures that the bullet slides on it's first move
        and     %00000011
        jr      nz,moveBulletCheck              ; only check slide every 4th frame
        ld      e,(ix+4)
        ld      d,(ix+5)                        ; DE = bullet y
        ld      a,(ix+7)
        dec     a
        jr      z,moveBulletCheck               ; don't slide if slide value is 1
        dec     a
        jr      z,bulletSlideUp
        jr      bulletSlideDown
removeBullet:
        ld      (ix+0),0                        ; disable this entry
        ret                                     ; ret to loop
bulletSlideUp:
        dec     de
        jr      bulletSlideSaveY
bulletSlideDown:
        inc     de
bulletSlideSaveY:
        ld      (ix+4),e
        ld      (ix+5),d
moveBulletCheck:
#ifdef TI83PLUS
        ld      a,(ix+0)                        ; A = bullet type
        dec     a
        add     a,a
        add     a,a
        add     a,a
        add     a,a                             ; A = offset to bullet type in bulletCentreTable/ebulletCentreTable
        ld      l,a
        ld      a,(ix+1)                        ; A = bullet direction
        add     a,a
        add     a,a
        add     a,l                             ; A = offset to bullet type & direction in bulletCentreTable/ebulletCentreTable
        ld      e,a
        ld      d,0
        ld      hl,(pCentreTable)               ; HL => either bulletCentreTable or ebulletCentreTable
#endif
#ifdef TI84PLUSCE
        ld      l,(ix+0)
        dec     l
        ld      h,24
        mlt     hl
        ld      e,(ix+1)
        ld      d,6
        mlt     de
        add     hl,de
        ld      de,(pCentreTable)
#endif
        add     hl,de                           ; HL => offsets to centre of bullet
#ifdef TI83PLUS
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = x offset to centre of bullet
        inc     hl                              ; HL => y offset to centre of bullet
#endif
#ifdef TI84PLUSCE
        call    _lddehl
#endif
        push    hl                              ; save ptr
#ifdef TI84PLUSCE
        or a \ sbc hl,hl
#endif
        ld      l,(ix+2)
        ld      h,(ix+3)                        ; HL = bullet x
        add     hl,de                           ; HL = x coord of centre of bullet
        ex      de,hl                           ; DE = x coord of centre of bullet
        pop     hl                              ; HL => y offset to centre of bullet
#ifdef TI83PLUS
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = y offset to centre of bullet
        ld      l,(ix+4)
        ld      h,(ix+5)                        ; HL = bullet y
        add     hl,bc                           ; HL = y coord of centre of bullet
#endif
#ifdef TI84PLUSCE
        push    de
        call    _lddehl
        or a \ sbc hl,hl
        ld      l,(ix+4)
        ld      h,(ix+5)
        add     hl,de
        pop     de
#endif
        call    getTile                         ; A = tile at bullet centre; HL => tile in level data
        push    hl                              ; save tile ptr
        ld      hl,fBulletCat
        bit     0,(hl)                          ; check if player or enemy bullet that's being moved
        pop     hl                              ; HL => tile
        jr      nz,ebulletTileCheck             ; if enemy bullet, do the simplified check
        cp      TILE_SOLID                      ; solid tile?
        ret     c                               ; if not, ret to loop without terminating bullet
        cp      TOW_INTEX_CONSOLE_UL
        jr      z,shootIntex
        cp      TOW_INTEX_CONSOLE_UL+1
        jr      z,shootIntex
        cp      TILE_POWERDOME                  ; power dome or door tile?
        push    ix                              ; save bullet data ptr
        call    nc,shootTile                    ; if so, run the tile shooting routine
        pop     ix                              ; restore bullet data ptr
        jr      removeBulletCheckAnim
shootIntex:
        push    ix
        call    shootTile
        pop     ix
removeBulletCheckAnim:
        ld      a,(ix+0)                        ; A = bullet type
        dec     a                               ; is it 0 (machine gun)?
        jr      nz,removeBullet                 ; if not, remove bullet without the explosion animation
        ; create an explosion animation
        ; first randomly pick an offset line from bulletWallAnimTable
#ifdef TI83PLUS
        ld      b,96
        call    getRandom                       ; A = 0-95
        and     %01110000                       ; A = 0,16,32,48,64,80
        ld      b,0
        ld      c,a                             ; BC = random line offset for bulletWallAnimTable
#endif
#ifdef TI84PLUSCE
        ld      b,6
        call    getRandom
        ld      c,a
        ld      b,24
        mlt     bc
#endif
#ifdef TI83PLUS
        ld      a,(ix+1)                        ; A = bullet direction
        add     a,a
        add     a,a
        ld      l,a
        ld      h,0                             ; HL = direction offset for bulletWallAnimTable
#endif
#ifdef TI84PLUSCE
        ld      l,(ix+1)
        ld      h,6
        mlt     hl
#endif
        add     hl,bc                           ; HL = total offset for bulletWallAnimTable
        ld      de,bulletWallAnimTable
        add     hl,de                           ; HL => offsets for animation x,y
#ifdef TI83PLUS
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = x offset
        inc     hl                              ; HL => y offset
#endif
#ifdef TI84PLUSCE
        call    _lddehl
#endif
        push    hl
#ifdef TI84PLUSCE
        ld      hl,0
#endif
        ld      l,(ix+2)
        ld      h,(ix+3)                        ; HL = bullet x
        add     hl,de
        ex      de,hl                           ; DE = x coord for explosion animation
        pop     hl                              ; HL => y offset
#ifdef TI83PLUS
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = y offset
#endif
#ifdef TI84PLUSCE
        push    de
        call    _lddehl
        push de \ pop bc
        pop     de
        ld      hl,0
#endif
        ld      l,(ix+4)
        ld      h,(ix+5)                        ; HL = bullet y
        add     hl,bc                           ; HL = y coord for explosion animation
        ld      bc,0*256+ANIM_EXPLOSION         ; B = animation type; C = initial counter
        call    newAnim
        jp      removeBullet
ebulletTileCheck:
        cp      TILE_SOLID                      ; solid tile hit?
        jp      nc,removeBullet                 ; if so, clear the enemy bullet entry
        ret

;------------------------------------------------
; shootTile - find location of a shootable tile that has been shot and damage/destroy it
;   input:  A = tile number
;           HL => tile in level data
;   output: none
;------------------------------------------------
shootTile:
        cp      TILE_DOOR                       ; check if a door tile
        ld      c,HITS_DOOR                     ; C = number of hits required to destroy a door
        jr      nc,doShootTile                  ; if it's a door, we can go straight ahead and process it
        ld      de,(levelFlags)                 ; E = level flags
        ; check if an intex terminal (and if it can be destroyed)
        cp      TOW_INTEX_CONSOLE_UL
        jr      z,checkIntex
        cp      TOW_INTEX_CONSOLE_UL+1
        jr      nz,checkPowerDome
        dec     hl                              ; HL => left intex tile
checkIntex:
        bit     LFLAG_INTEX,e
        ret     z
        ld      c,HITS_INTEX
        jr      doShootTile
checkPowerDome:
        ; otherwise, it's a power dome, which is 2x2 tiles in size, so first we need to
        ; calc a ptr to the top-left tile of the power dome
        bit     LFLAG_POWERDOMES,e              ; check if power domes can be shot
        ret     z                               ; if not, just leave
        ld      c,HITS_POWERDOME                ; C = number of hits req'd to destroy a power dome
        sub     TILE_POWERDOME+2                ; check if it's the 3rd or 4th tile of a power dome
        jr      c,shootPowerDomeYOK             ; if not, HL is pointing to the correct row (y coord) in level data
#ifdef TI83PLUS
        ld      de,(levelWidth)
        ld      d,0                             ; DE = level width (in blocks)
#endif
#ifdef TI84PLUSCE
        push    af
        ld      a,(levelWidth)
        ld de,0 \ ld e,a
        pop     af
#endif
        or      a                               ; clear carry
        sbc     hl,de                           ; HL => either tile 1 or 2 of the power dome
shootPowerDomeYOK:
        bit     0,a                             ; check if it's the 2nd or 4th tile of a power dome
        jr      z,doShootTile                   ; if not, HL is pointing to the correct tile
        dec     hl                              ; HL => 1st (top left) tile of power dome
doShootTile:
        ld      de,(tileShotPtr)                ; DE => previously shot tile in level data (NULL if none)
        CPHLDE()                                ; compare pointers to see if this tile is the same as the last one shot
        jr      z,updateShotTile                ; if so, go straight to update
        ld      a,c                             ; A = number of hits req'd to destroy tile (C was set earlier)
        ld      (tileShotHits),a                ; set number of hits left (ie. tile health)
updateShotTile:
        ld      (tileShotPtr),hl                ; save ptr to show tile that has been shot this frame
        ld      a,(tileShotHits)                ; tile hits (health)
        dec     a
        ld      (tileShotHits),a                ; save new hits (health)
        or      a                               ; check if it has reached zero
        ret     nz                              ; if not, simply leave
        ; otherwise, tile has been "destroyed", and thus needs to be updated in level data to a new tile or new set of tiles
        ld      de,$0000
        ld      (tileShotPtr),de                ; reset ptr to NULL to indicate that no tile has previously been shot
        ; create explosion animation
        push    hl
        call    getExactXYFromHL
        inc de \ inc de \ inc de
        inc hl \ inc hl
        ld      bc,0*256+ANIM_EXPLOSION
        call    newAnim
        pop     hl
        ld      a,(hl)                          ; A = tile that has been "destroyed"
        cp      TILE_DOOR                       ; is it a door?
        jr      nc,shotOpenDoor                 ; if so, handle that
        ; check if it's an intex terminal
        cp      TOW_INTEX_CONSOLE_UL
        jr      z,intexDestroyed
        ; otherwise, it's a power dome, which is a little more complicated
        call    destroy2PDTiles                 ; destroy 1st & 2nd tile
#ifdef TI83PLUS
        ld      de,(levelWidth)
        ld      d,0
        dec     e
#endif
#ifdef TI84PLUSCE
        ld      a,(levelWidth)
        dec     a
        ld de,0 \ ld e,a
#endif
        add     hl,de                           ; HL => 3rd tile
        call    destroy2PDTiles                 ; destroy 3rd & 4th tile
        jp      trgPowerDome
destroy2PDTiles:
        ld      a,(hl)                          ; A = power dome tile
        sub     4                               ; A = destroyed power dome tile
        call    putTile                         ; place it & draw it
        inc     hl                              ; HL => next tile
        ld      a,(hl)                          ; repeat
        sub     4
        jp      putTile
shotOpenDoor:
        push    af
        push    hl
        ; set the achievement
        ld      c,ACH_SHOTDOOR_ID
        ld      b,ACH_SHOTDOOR_BIT
        ld      de,ACH_SHOTDOOR_OFFSET
        ld      hl,viewNewAchievementSaveScr
        bcall(ab5_setAchievement)
        pop     hl
        pop     af
        jp      openDoor                        ; open the door and ret
intexDestroyed:
        sub     2
        call    putTile
        inc     hl
        ld      a,(hl)
        sub     2
        call    putTile
        jp      trgIntex

;------------------------------------------------
; doEbullet - update enemy bullet or create one if there is an enemy ready to shoot
;   input:  none
;   output: none
;------------------------------------------------
doEbullet:
        ld      a,(ebullet)
        or      a
        jp      nz,moveEbullet                  ; if there is an active bullet, move it
        ld      l,%00111111                     ; bitmask
        ld      a,(e1cheatFlags)
        bit     insaneModeCheat,a
        jr      z,debAfterInsane
        srl     l
        srl     l                               ; L = %00001111, ie. shoot every 16th frame in INSANE
        jr      debAfterTough
debAfterInsane:
        ld      a,(difficulty)
        or      a
        jr      z,debAfterTough                 ; shoot every 64th frame in NORMAL
        srl     l                               ; shoot every 32nd frame in TOUGH
debAfterTough:
        ld      a,(frame)
        and     l
        ret     nz                              ; can only shoot a new enemy bullet every (L+1)th frame
        ; check enemyTable to see if any of the enemies can shoot
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
findShootingEnemy:
#ifdef TI83PLUS
        ld      l,(ix)
        dec     l                               ; L = enemy type {0-X} or -1 for no enemy
        bit     7,l
        jr      nz,tryNextShootingEnemy         ; if no enemy in this entry, skip to next
        ld      h,0
#endif
#ifdef TI84PLUSCE
        ld      a,(ix)
        dec     a
        bit     7,a
        jr      nz,tryNextShootingEnemy
        LDHLA()
#endif
        ld      de,enemyBulletType
        add     hl,de
        ld      a,(hl)
        or      a
        jr      z,tryNextShootingEnemy          ; if an enemy doesn't have a bullet type, an enemy can't shoot
        ; create enemy bullet
        ld      (ebullet),a                     ; save type
        LDHLA()
        dec     l                               ; HL = enemy bullet type {0-X}
        ld      a,(ix+1)
        and     %00000011                       ; {0-3}
        ld      (ebullet+1),a                   ; save direction
#ifdef TI83PLUS
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; HL = offset to type in ebulletOffsetTable
        add     a,a
        add     a,a
        ld      e,a
        ld      d,0                             ; DE = direction * 2
#endif
#ifdef TI84PLUSCE
        ld      h,24
        mlt     hl
        ld      e,a
        ld      d,6
        mlt     de
#endif
        add     hl,de
        ld      de,ebulletOffsetTable
        add     hl,de                           ; HL => x offset
        push    hl
        LDHLIND()
        ld      e,(ix+2)
        ld      d,(ix+3)
#ifdef TI84PLUSCE
        call    _setHLUto0
        call    _setDEUto0
#endif
        add     hl,de                           ; HL = x coord
#ifdef TI83PLUS
        ld      (ebullet+2),hl                  ; save it
#endif
#ifdef TI84PLUSCE
        ld      a,l
        ld      (ebullet+2),a
        ld      a,h
        ld      (ebullet+3),a
#endif
        pop     hl
#ifdef TI84PLUSCE
        inc     hl
#endif
        inc     hl
        inc     hl                              ; HL => y offset
        LDHLIND()
        ld      e,(ix+4)
        ld      d,(ix+5)
#ifdef TI84PLUSCE
        call    _setHLUto0
        call    _setDEUto0
#endif
        add     hl,de                           ; HL = y coord
#ifdef TI83PLUS
        ld      (ebullet+4),hl                  ; save it
#endif
#ifdef TI84PLUSCE
        ld      a,l
        ld      (ebullet+4),a
        ld      a,h
        ld      (ebullet+5),a
#endif
        ld      a,1
        ld      (ebullet+7),a
        ret
tryNextShootingEnemy:
        ld      de,ENEMY_SIZE
        add     ix,de
        djnz    findShootingEnemy
        ret
moveEbullet:
        ld      a,1
        ld      (fBulletCat),a                  ; set bullet category to "enemy"
        ld      hl,ebulletCentreTable
        ld      (pCentreTable),hl               ; set pointer value
        ld      ix,ebullet
        ld      l,(ix+1)
#ifdef TI83PLUS
        ld      h,0                             ; HL = direction
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,3
        mlt     hl
#endif
        ld      de,bulletMoveTable
        add     hl,de
        LDHLIND()                               ; HL => routine to move bullet
        jp      (hl)                            ; jump to moving routine and ret

;------------------------------------------------
; drawEbullet - draw enemy bullet
;   input:  none
;   output: none
;------------------------------------------------
drawEbullet:
        ld      a,(ebullet)
        or      a
        ret     z                               ; if no enemy bullet, leave
        dec     a                               ; A = enemy bullet type {0-X}
#ifdef TI83PLUS
        add     a,a                             ; x2
        ld      e,a
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      e,a
        ld      d,3
        mlt     de
#endif
        ld      hl,ebulletSpriteTable
        add     hl,de
#ifdef TI83PLUS
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE => sprite set
#endif
#ifdef TI84PLUSCE
        ld      de,(hl)
#endif
        ld      hl,(ebullet+1)
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl                           ; HL = direction * 8
#endif
#ifdef TI84PLUSCE
        ld      h,128
        mlt     hl
        add     hl,hl
#endif
        add     hl,de                           ; HL => bullet sprite
        ld      ix,ebullet
        call    getBulletScreenCoords           ; B = x coord, C = y coord (on screen)
#ifdef TI83PLUS
        jp      putClippedSprite                ; draw it and leave
#endif
#ifdef TI84PLUSCE
        ld      e,b
        ld      d,c
        jp      drawClippedTransSprite
#endif

;------------------------------------------------
; getBulletScreenCoords - get screen coords for a bullet
;   input:  IX => bullet entry (can be either player or enemy)
;   output: B = x coord
;           C = y coord
;   notes:  preserves HL
;------------------------------------------------
getBulletScreenCoords:
        push    hl
        ld      de,(screenX)
        or      a
#ifdef TI84PLUSCE
        sbc     hl,hl
#endif
        ld      l,(ix+2)
        ld      h,(ix+3)                        ; HL = bullet x
        sbc     hl,de                           ; HL = x coord to display at
        ld      b,l                             ; B = x coord to display at
        ld      de,(screenY)
        or      a
#ifdef TI84PLUSCE
        sbc     hl,hl
#endif
        ld      l,(ix+4)
        ld      h,(ix+5)                        ; HL = bullet y
        sbc     hl,de                           ; HL = y coord to display at
        ld      c,l                             ; C = y coord to display at
        pop     hl
        ret

.end
